/*
 =========================================================================================================
 C. Voglis, Department of Computer Science,
 Univeristy of Ioannina, Greece

 File: 				SOURCE_main.h
 Usage: 			Main function definitions
 Last Modified: 	21/08/2013
 =========================================================================================================
*/

#ifndef _SOURCE_MAIN_H_
#define _SOURCE_MAIN_H_


 /**
 *  Returns the bounds of the objective function.
 */
extern void Bounds_F(double *xl, double *xr, int N);

/**
 * Writes preamble information on the console.
 */
extern void SPreamble(ALGO_par algo, PROBLEM_par prob, int algi);

/**
 * Writes preamble information in the designated output file.
 */
extern void FPreamble(FILE *fout1, ALGO_par algo, PROBLEM_par prob, int algi);

/**
 * Runs the actual minimization method.
 */
extern void UPSO_DE(double swarm[], double fswarm[], double vel[], double bestpos[],
		double fbestpos[], ALGO_par algo, PROBLEM_par prob, RUN_STATS *stats, int algi);

/**
 * Parses command line arguments of mempsode command.
 */
int parse_arguments(int argc, char **argv, int *algi, PROBLEM_par *prob, ALGO_par *algo,
		int *PSOparset, char *foutpre, int *noprint, int *printsolflag);

/**
 * Special parser for Torczon's MDS optimization method
 */
extern void parse_torczon_args(char *args, Torczon *torc);

/**
 * Special parser for BFGS optimization method
 */
extern void parse_bfgs_args(char *args, Bfgs *torc);

/**
 * Writes the results of a full run in the appropriate files and the console
 */
void write_results(int i, PROBLEM_par prob, RUN_STATS stats, FILE *fout1, FILE *fout2, FILE *fout3,
		int noprint);

/**
 * Writes PSO position and velocity parameters to file for later retrieval
 */
extern void dump_swarm(double *swarm, double *fswarm, double *bestpos, double *fbestpos, double *vel, ALGO_par algo, PROBLEM_par pr, RUN_STATS stats);

/**
 * Retrieves PSO position and velocity  parameters from a file
 */
extern void read_swarm(double *swarm, double *fswarm, double *bestpos, double *fbestpos, double *vel, ALGO_par algo, PROBLEM_par pr, RUN_STATS *stats);

int noprint, helpflag, printsolflag;
int aa;

struct {
	int iflag;
} pflag_;

extern struct{
	int ipro;
} funcid_;

#endif
