/*
 =========================================================================================================
 C. Voglis, Department of Computer Science, 
 Univeristy of Ioannina, Greece

 File: 				SOURCE_print.c
 Usage: 			Prints partial and final output to console and files
 Last Modified: 	21/08/2013
 =========================================================================================================
 */

 #include <stdlib.h>
 #include "SOURCE_params.h"

 /*
 ----------------------------
 WRITING PREAMBLE ON SCREEN 
 ----------------------------

 Input: 
 ALGO_par 			algo 	- A structure that contains parameters for the algorithm PSO/DE
 PROBLEM_par 		prob 	- A structure that contains test problem depended parameters
 int 				algi    - 1 for PSO / 0 for DE
 Output:
 <none>
 */
void SPreamble(ALGO_par algo, PROBLEM_par prob, int algi) {
	int j;

	printf("PROBLEM PARAMETERS\n");
	printf("Problem   : %i \n", prob.aa);
	printf("Dimension : %i \n", prob.dim);
	printf("NumOfExp  : %i \n", prob.nexp);
	printf("Seed  	  : %i \n", algo.seed);
	if (prob.maxit != -1)
		printf("MaxIter   : %li \n", prob.maxit);
	else
		printf("MaxIter   : inf \n");
	if (prob.maxfev != -1)
		printf("MaxFev    : %li \n", prob.maxfev);
	else
		printf("MaxFev    : inf \n");
	if (prob.maxgev != -1)
		printf("MaxGev    : %li \n", prob.maxgev);
	else
		printf("MaxGev    : inf \n");
	if (prob.iVTR != -1)
		printf("Target    : %lf \n", prob.VTR);
	else
		printf("Target    : -inf \n");

	printf("Xmin      : [ ");
	for (j = 0; j < prob.dim; j++)
		printf("%.3lf ", prob.Xmin[j]);
	printf("]\n");
	printf("Xmax      : [ ");
	for (j = 0; j < prob.dim; j++)
		printf("%.3lf ", prob.Xmax[j]);
	printf("]\n\n");

	if (algi == 1) {
		printf("UPSO PARAMETERS\n");
		printf("Memetic   : %i \n", algo.memetic);
		printf("UF        : %.3lf \n", algo.UF);
		printf("Vscale    : %.3lf \n", algo.Vscale);
		printf("Prob      : %.3lf \n", algo.prob);
		printf("x, c1, c2 : %.3lf %.3lf %.3lf \n", algo.chi, algo.c1, algo.c2);
		printf("Nradius   : %i \n", algo.nradius);
		printf("R3use     : %i \n", algo.R3use);
		printf("R3mean    : %.2lf \n", algo.R3mean);
		printf("R3std     : %.2lf \n", algo.R3std);
	} else {
		printf("DE PARAMETERS\n");
		printf("OPER      : %i \n", algo.oper);
		printf("F         : %.2lf \n", algo.F);
		printf("CR        : %.2lf \n", algo.CR);
	}
	printf("SS        : %i \n", algo.SS);
	printf(
			"====================================================================================================================\n");
	printf("%3s - %s - %12s - %7s - %8s - %5s - %5s - %34s - %6s - %8s \n", "EXP", "S",
			"   F(SOL)   ", " ITER ", " FEVALS", "LOCAL", "GEVALS", "         LAST HIT           ",
			" BPUPD", "CPU TIME");
	printf("                                                                %7s,%8s,%7s,%7s\n\n",
			" ITER ", " FEVALS", "GEVALS", "LOCAL");
}

/*
 =========================================================================================================
 =========================================================================================================

 --------------------------
 WRITING PREAMBLE IN FILE 
 --------------------------

 Input: 
 FILE 			*fout	- The FILE structure of the output file
 ALGO_par 		algo 	- A structure that contains parameters for the algorithm PSO/DE
 PROBLEM_par 	prob 	- structure that contains problem depended parameters
 int 			algi    - 1 for PSO / 0 for DE
 Output:
 <none>
 */

void FPreamble(FILE *fout1, ALGO_par algo, PROBLEM_par prob, int algi) {

	int j;

	fprintf(fout1,
			"=============================================================================================\n");
	fprintf(fout1, "PROBLEM PARAMETERS\n");
	fprintf(fout1, "Problem   : %d\n", prob.aa);
	fprintf(fout1, "Dimension : %d\n", prob.dim);
	fprintf(fout1, "NumOfExp  : %d\n", prob.nexp);
	fprintf(fout1, "Seed      : %i \n", algo.seed);
	if (prob.maxit != -1)
		fprintf(fout1, "MaxIter   : %li \n", prob.maxit);
	else
		fprintf(fout1, "MaxIter   : inf \n");
	if (prob.maxfev != -1)
		fprintf(fout1, "MaxFev    : %li \n", prob.maxfev);
	else
		fprintf(fout1, "MaxFev    : inf \n");
	if (prob.maxgev != -1)
		fprintf(fout1, "MaxGev    : %li \n", prob.maxgev);
	else
		fprintf(fout1, "MaxGev    : inf \n");
	if (prob.iVTR != -1)
		fprintf(fout1, "Target    : %lf \n", prob.VTR);
	else
		fprintf(fout1, "Target    : -inf \n");

	fprintf(fout1, "Xmin      : [");
	for (j = 0; j < prob.dim; j++)
		fprintf(fout1, "%le ", prob.Xmin[j]);
	fprintf(fout1, "]\n");
	fprintf(fout1, "Xmax      : [");
	for (j = 0; j < prob.dim; j++)
		fprintf(fout1, "%le ", prob.Xmax[j]);
	fprintf(fout1, "]\n\n");

	if (algi == 1) {
		fprintf(fout1, "UPSO PARAMETERS\n");
		fprintf(fout1, "Memetic   : %d\n", algo.memetic);
		fprintf(fout1, "UF        : %3.1f\n", algo.UF);
		fprintf(fout1, "Vscale   : %.3lf \n", algo.Vscale);
		fprintf(fout1, "Prob     : %.3lf \n", algo.prob);
		fprintf(fout1, "x, c1, c2 : %5.3f %4.2f %4.2f\n", algo.chi, algo.c1, algo.c2);
		fprintf(fout1, "Nradius   : %d\n", algo.nradius);
		fprintf(fout1, "R3use     : %d\n", algo.R3use);
		fprintf(fout1, "R3mean    : %f\n", algo.R3mean);
		fprintf(fout1, "R3std     : %f\n", algo.R3std);
	} else {
		fprintf(fout1, "DE PARAMETERS\n");
		fprintf(fout1, "OPER      : %d\n", algo.oper);
		fprintf(fout1, "F         : %f\n", algo.F);
		fprintf(fout1, "CR        : %f\n", algo.CR);
	}
	fprintf(
			fout1,
			"====================================================================================================================\n");
	fprintf(fout1, "%3s - %s - %12s - %7s - %8s - %5s - %5s - %33s - %6s - %8s \n\n", "EXP", "S",
			"   F(SOL)   ", " ITER ", " FEVALS", "LOCAL", "GEVALS", "    LAST HIT              ",
			" BPUPD", "CPU TIME");
}

void dump_swarm(double *swarm, double *fswarm, double *bestpos, double *fbestpos, double *vel,
		ALGO_par algo, PROBLEM_par prob, RUN_STATS stats) {
	FILE *fp;

	if ((fp = fopen(TMPOUFILE, "w")) == NULL) {
		printf("Cannot open %s for writing \n", TMPOUFILE);
		exit(0);
	} else {
		// 
		// Write the size of the swarm and the dimensionality of the problem
		fwrite(&algo.SS, sizeof(int), 1, fp);
		fwrite(&prob.dim, sizeof(int), 1, fp);
		// 
		// Write 1:for PSO and 2: for DE 
		fwrite(&algo.algi, sizeof(int), 1, fp);
		if (algo.algi == 1) {
			//
			// PSO case: Dump swarm, fswarm, bestpos, fbestpos
			fwrite(swarm, sizeof(double), algo.SS * prob.dim, fp);
			fwrite(fswarm, sizeof(double), algo.SS, fp);
			fwrite(bestpos, sizeof(double), algo.SS * prob.dim, fp);
			fwrite(fbestpos, sizeof(double), algo.SS, fp);
			//
			// PSO case: Dump velocity
			fwrite(vel, sizeof(double), algo.SS * prob.dim, fp);
		} else {
			//
			// DE case: Dump  bestpos, fbestpos
			fwrite(swarm, sizeof(double), algo.SS * prob.dim, fp);
			fwrite(fswarm, sizeof(double), algo.SS, fp);
			fwrite(bestpos, sizeof(double), algo.SS * prob.dim, fp);
			fwrite(fbestpos, sizeof(double), algo.SS, fp);
		}
		fwrite(&(stats.bestind), sizeof(int), 1, fp);
		fwrite(&(stats.bestval), sizeof(double), 1, fp);
		fwrite(stats.best_x, sizeof(double), prob.dim, fp);
		fwrite(&(stats.best_f), sizeof(double), 1, fp);
	}
	fclose(fp);

}

void read_swarm(double *swarm, double *fswarm, double *bestpos, double *fbestpos, double *vel,
		ALGO_par algo, PROBLEM_par prob, RUN_STATS *stats) {
	FILE *fp;
	int SS, dim, algi;

	if ((fp = fopen(TMPINFILE, "r")) == NULL) {
		printf("Cannot open %s for reading swarm parameters \n", TMPINFILE);
		exit(0);
	} else {
		// 
		// Write the size of the swarm and the dimensionality of the problem
		fread(&SS, sizeof(int), 1, fp);
		fread(&dim, sizeof(int), 1, fp);
		// 
		// Write 1:for PSO and 2: for DE 
		fread(&algi, sizeof(int), 1, fp);

		if (SS != algo.SS || dim != prob.dim || algi != algo.algi) {
			printf("---------------------------------\n");
			printf("---   Incompatible dump file !  --\n");
			printf("---   Skipping initialization  --\n");
			printf("---------------------------------\n");
			exit(0);
			return;
		} else {
			if (algo.algi == 1) {
				//
				// PSO case: Dump swarm, fswarm, bestpos, fbestpos
				fread(swarm, sizeof(double), algo.SS * prob.dim, fp);
				fread(fswarm, sizeof(double), algo.SS, fp);
				fread(bestpos, sizeof(double), algo.SS * prob.dim, fp);
				fread(fbestpos, sizeof(double), algo.SS, fp);
				//
				// PSO case: Dump velocity
				fread(vel, sizeof(double), algo.SS * prob.dim, fp);

			} else {
				//
				// DE case: Dump  bestpos, fbestpos
				fread(swarm, sizeof(double), algo.SS * prob.dim, fp);
				fread(fswarm, sizeof(double), algo.SS, fp);
				fread(bestpos, sizeof(double), algo.SS * prob.dim, fp);
				fread(fbestpos, sizeof(double), algo.SS, fp);
			}
		}
		fread(&(stats->bestind), sizeof(int), 1, fp);
		fread(&(stats->bestval), sizeof(double), 1, fp);
		fread(stats->best_x, sizeof(double), prob.dim, fp);
		fread(&(stats->best_f), sizeof(double), 1, fp);
	}
	fclose(fp);
}

void print_swarm(double swarm[], int n, int m) {
	int i, j;

	printf("------------------------------ \n");
	for (i = 0; i < m; i++) {
		printf("Swarm %i: ", i);
		for (j = 0; j < n; j++)
			printf(" %f ", swarm[i * n + j]);
		printf("\n");
	}

}
