/*
 =========================================================================================================
 C. Voglis, Department of Computer Science,
 Univeristy of Ioannina, Greece

 File: 				SOURCE_problems.c
 Usage: 			Implementation of test function collection
 Last Modified: 	21/08/2013
 =========================================================================================================
 */
//=========================================================================================================
//=========================================================================================================
/*
 B E N C H M A R K    O P T I M I Z A T I O N    P R O B L E M S

 Currently Implemented:
 0. 	SPHERE, X=[0,...,0], F=0, [-100,100]
 1. 	ROSENBROCK, X=[1,...,1], F=0, [-30,30]
 2.	RASTRIGIN, X=[0,...,0], F=0, [-5.12,5.12]
 3.	GRIEWANK, X=[0,...,0], F=0, [-600,600]
 4.	ACKLEY, X=[0,...,0], F=0, [-20,30]
 5. 	FREUDENSTEIN-ROTH, X=[5,4], F=0
 6.	LEVY No.3, 18 global minimizers with F=-176.542
 7.	LEVY, X=[1,...,1], F=0, DIM=3,4,5,8,10
 8.	POWELL BADLY SCALED, X=[1.098*10^(-5),9.106], F=0
 9	WOOD, X=[1,1,1,1], F=0
 10.	TRIGONOMETRIC, F=0
 11.	SCHAFFER F6, X=[0,...,0], F=0, [-100,100]
 12. 	SCHWEFEL'S DOUBLE SUM, X=[0,...,0], F=0, [-65.536, 65.536]
 13.	SCHWEFEL FUNCTION
 14.	Nonlinear equations: Interval Arithmetic Benchmark,  dim = 10, F = 0, [-2, 2]
 15.	Nonlinear equations: Neurophysiology Application Benchmark dim = 6, F = 0, [-10, 10]
 16.	Nonlinear equations: Chemical Equilibrium Application dim = 5, F = 0, [-10, 10]
 17.	Nonlinear equations: Kinematic Application dim = 8, F = 0,  [-10, 10]
 18.	Nonlinear equations: Combustion Application dim = 10, F = 0,  [-10. 10]
 19.	Nonlinear equations: Economics Modelling Application,  dim = arbitrary, F = 0, [-10, 10]
 20.	Shifted Sphere problem F = -450 [-100, 100]
 21.	Shifted Rosenbrock problem F = 390, [-100, 100]
 22.	Shifted Rastrigin problem  F = -330, [-5, 5]
 23.	Shifted Griewank problem F = -180, [-600, 600]
 24.	Shifted Ackley problem  F = -140, [-32, 32]
 25.	Shifted Schwefel problem F = -450, [-100, 100]
 26.	Bird test function problem n = 2, F = -106.765, [-50, 50]
 27.	Bohachevsky test function problem n = 2, F = 0. [-50, 50]
 28.	Carrom table test function problem n = 2, F = 0. [-100, 100]
 29.	Guillin hills test function problem F = ??, [0, 1]
 30.	Holder test function problem F = ??, [-100, 100]
 31.	Lagermann test function problem F = ??, [0, 100]
 32.	Lennard Jones Energy function

 */
//=========================================================================================================
//=========================================================================================================
#include <math.h>
#include <stdlib.h>
#include "SOURCE_params.h"
#include "SOURCE_problems.h"
#include "objective.h"

double lj(double[], int);
double ljpot(double);

//----------------------------
// Sample objective functions
//----------------------------
void Objective_F(double X[], int N, double *F) {
	int i, j, l, k;
	double tmp1, tmp2[N], sum;
	double pi = 3.14159265358979;
	int nterms;
	double f[20];
	double R, R5, R6, R7, R8, R9, R10;
	double a[17][4] = { { -0.249150680, 0.125016350, -0.635550077, 1.48947730 }, { 1.609135400,
			-0.686607360, -0.115719920, 0.23062341 }, { 0.279423430, -0.119228120, -0.666404480,
			1.32810730 }, { 1.434801600, -0.719940470, 0.110362110, -0.25864503 }, { 0.000000000,
			-0.432419270, 0.290702030, 1.16517200 }, { 0.400263840, 0.000000000, 1.258776700,
			-0.26908494 }, { -0.800527680, 0.000000000, -0.629388360, 0.53816987 }, { 0.000000000,
			-0.864838550, 0.581404060, 0.58258598 }, { 0.074052388, -0.037157270, 0.195946620,
			-0.20816985 }, { -0.083050031, 0.035436896, -1.228034200, 2.68683200 }, { -0.386159610,
			0.085383482, 0.000000000, -0.69910317 }, { -0.755266030, 0.000000000, -0.079034221,
			0.35744413 }, { 0.504201680, -0.039251967, 0.026387877, 1.24991170 }, { -1.091628700,
			0.000000000, -0.057131430, 1.46773600 }, { 0.000000000, -0.432419270, -1.162808100,
			1.16517200 }, { 0.049207290, 0.000000000, 1.258776700, 1.07633970 }, { 0.049207290,
			0.013873010, 2.162575000, -0.69686809 } };

	/////////////////////////////////////////////////////////////
	FEVALCOUNT;
	/////////////////////////////////////////////////////////////

	// SPHERE, X=[0,...,0], F=0, [-100,100]
	//--------------------------------------
	if (funcid_.ipro == 0) {

		*F = 0.0;
		for (i = 0; i < N; i++)
			*F = *F + pow(X[i], 2);

	}

	// ROSENBROCK, X=[1,...,1], F=0, [-30,30]
	//----------------------------------------
	else if (funcid_.ipro == 1) {
		*F = 0.0;
		for (i = 0; i < N - 1; i++)
			*F = *F + 100.0 * pow((X[i + 1] - X[i] * X[i]), 2) + pow((X[i] - 1.0), 2);
	}

	// RASTRIGIN, X=[0,...,0], F=0, [-5.12,5.12]
	//-------------------------------------------
	else if (funcid_.ipro == 2) {
		*F = 0.0;
		for (i = 0; i < N; i++)
			*F = *F + pow(X[i], 2) + 10.0 - 10.0 * cos(2 * pi * X[i]);
	}

	// GRIEWANK, X=[0,...,0], F=0, [-600,600]
	//----------------------------------------
	else if (funcid_.ipro == 3) {
		tmp1 = 0.0;
		for (i = 0; i < N; i++)
			tmp1 = tmp1 + pow(X[i], 2);
		*F = tmp1 / 4000.0;
		tmp1 = 1.0;
		for (i = 0; i < N; i++)
			tmp1 = tmp1 * cos(X[i] / sqrt((double) i + 1.0));
		*F = *F - tmp1 + 1.0;
	}

	// ACKLEY, X=[0,...,0], F=0, [-20,30]
	//------------------------------------
	else if (funcid_.ipro == 4) {
		*F = 20.0 + exp(1.0);
		tmp1 = 0.0;
		for (i = 0; i < N; i++)
			tmp1 = tmp1 + X[i] * X[i];
		tmp1 = sqrt(tmp1 / N);
		*F = *F - 20.0 * exp(-0.2 * tmp1);
		tmp1 = 0.0;
		for (i = 0; i < N; i++)
			tmp1 = tmp1 + cos(2.0 * pi * X[i]);
		tmp1 = tmp1 / N;
		*F = *F - exp(tmp1);
	}

	// FREUDENSTEIN-ROTH, X=[5,4], F=0
	//---------------------------------
	else if (funcid_.ipro == 5) {
		tmp1 = (-13.0 + X[0] + ((5.0 - X[1]) * X[1] - 2.0) * X[1]);
		*F = tmp1 * tmp1;
		tmp1 = (-29.0 + X[0] + ((X[1] + 1.0) * X[1] - 14.0) * X[1]);
		*F = *F + tmp1 * tmp1;
	}

	// LEVY No.3, 18 global minimizers with F=-176.542
	//-------------------------------------------------
	else if (funcid_.ipro == 6) {
		tmp1 = 0.0;
		for (i = 0; i < 5; i++)
			tmp1 = tmp1 + (i + 1.0) * cos(i * X[0] + i + 1.0);
		*F = tmp1;
		tmp1 = 0.0;
		for (i = 0; i < 5; i++)
			tmp1 = tmp1 + (i + 1.0) * cos((i + 2.0) * X[1] + i + 1.0);
		*F = *F * tmp1;
	}

	// LEVY, X=[1,...,1], F=0, DIM=3,4,5,8,10
	//----------------------------------------
	else if (funcid_.ipro == 7) {
		for (i = 0; i < N; i++)
			tmp2[i] = 1.0 + (X[i] - 1.0) / 4.0;
		tmp1 = 0.0;
		*F = sin(pi * tmp2[0]) * sin(pi * tmp2[0]) + (tmp2[N - 1] - 1.0) * (tmp2[N - 1] - 1.0);
		for (i = 0; i < (N - 1); i++) {
			tmp1 = tmp2[i] - 1.0;
			tmp1 = tmp1 * tmp1 * (1.0 + 10.0 * sin(pi * tmp2[i + 1]) * sin(pi * tmp2[i + 1]));
			*F = *F + tmp1;
		}
	}

	// POWELL BADLY SCALED, X=[1.098*10^(-5),9.106], F=0
	//---------------------------------------------------
	else if (funcid_.ipro == 8) {
		tmp1 = 10000.0 * X[0] * X[1] - 1.0;
		*F = tmp1 * tmp1;
		tmp1 = exp(-X[0]) + exp(-X[1]) - 1.0001;
		*F = *F + tmp1 * tmp1;
	}

	// WOOD, X=[1,1,1,1], F=0
	//------------------------
	else if (funcid_.ipro == 9) {
		tmp1 = 10.0 * (X[1] - X[0] * X[0]);
		*F = tmp1 * tmp1;
		tmp1 = (1 - X[0]);
		*F = *F + tmp1 * tmp1;
		tmp1 = (X[3] - X[2] * X[2]);
		*F = *F + 90.0 * tmp1 * tmp1;
		tmp1 = (1 - X[2]);
		*F = *F + tmp1 * tmp1;
		tmp1 = (X[1] + X[3] - 2.0);
		*F = *F + 10.0 * tmp1 * tmp1;
		tmp1 = (X[1] - X[3]);
		*F = *F + 0.1 * tmp1 * tmp1;
	}

	// TRIGONOMETRIC, F=0
	//--------------------
	else if (funcid_.ipro == 10) {
		tmp1 = 0.0;
		for (i = 0; i < N; i++)
			tmp1 = tmp1 + cos(X[i]);
		*F = 0.0;
		for (i = 0; i < N; i++)
			tmp2[i] = N - tmp1 + (i + 1.0) * (1.0 - cos(X[i])) - sin(X[i]);
		for (i = 0; i < N; i++)
			*F = *F + tmp2[i] * tmp2[i];
	}

	// SCHAFFER F6, X=[0,...,0], F=0, [-100,100]
	//-------------------------------------------
	else if (funcid_.ipro == 11) {
		tmp1 = sin(sqrt(X[0] * X[0] + X[1] * X[1]));
		tmp1 = tmp1 * tmp1 - 0.5;
		*F = tmp1;
		tmp1 = 1.0 + 0.001 * (X[0] * X[0] + X[1] * X[1]);
		tmp1 = tmp1 * tmp1;
		*F = 0.5 + *F / tmp1;
	}

	// SCHWEFEL'S DOUBLE SUM, X=[0,...,0], F=0, [-65.536, 65.536]
	//------------------------------------------------------------
	else if (funcid_.ipro == 12) {
		*F = 0.0;
		for (i = 0; i < N; i++) {
			tmp1 = 0.0;
			for (j = 0; j <= i; j++)
				tmp1 = tmp1 + X[j];
			*F = *F + tmp1 * tmp1;
		}
	}

	// SCHWEFEL FUNCTION
	//-------------------
	else if (funcid_.ipro == 13) {
		*F = 418.9829 * N;
		tmp1 = 0.0;
		for (j = 0; j < N; j++)
			tmp1 += X[j] * sin(sqrt(fabs(X[j])));
		*F = *F + tmp1;
	}
	// Nonlinear equations Interval Arithmetic Benchmark
	// dim = 10 [-2, 2]
	//-------------------
	else if (funcid_.ipro == 14) {
		nterms = 10;
		f[0] = X[0] - 0.25428722 - 0.18324757 * X[3] * X[2] * X[8];
		f[1] = X[1] - 0.37842197 - 0.16275449 * X[0] * X[9] * X[5];
		f[2] = X[2] - 0.27162577 - 0.16955071 * X[0] * X[1] * X[9];
		f[3] = X[3] - 0.19807914 - 0.15585316 * X[6] * X[0] * X[5];
		f[4] = X[4] - 0.44166728 - 0.19950920 * X[6] * X[5] * X[2];
		f[5] = X[5] - 0.14654113 - 0.18922793 * X[7] * X[4] * X[9];
		f[6] = X[6] - 0.42937161 - 0.21180486 * X[1] * X[4] * X[7];
		f[7] = X[7] - 0.07056438 - 0.17081208 * X[0] * X[6] * X[5];
		f[8] = X[8] - 0.34504906 - 0.19612740 * X[9] * X[5] * X[7];
		f[9] = X[9] - 0.42651102 - 0.21466544 * X[3] * X[7] * X[0];

		(*F) = 0.0;
		for (j = 0; j < nterms; j++) {
			(*F) = (*F) + fabs(f[j]);
		}

	}
	// Nonlinear equations Neurophysiology Application Benchmark
	// dim = 6 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 15) {
		nterms = 6;
		f[0] = X[0] * X[0] + X[2] * X[2] - 1.0;
		f[1] = X[1] * X[1] + X[3] * X[3] - 1.0;
		f[2] = X[4] * X[2] * X[2] * X[2] + X[5] * X[3] * X[3] * X[3];
		f[3] = X[4] * X[0] * X[0] * X[0] + X[5] * X[1] * X[1] * X[1];
		f[4] = X[4] * X[0] * X[2] * X[2] + X[5] * X[3] * X[3] * X[1];
		f[5] = X[4] * X[0] * X[0] * X[2] + X[5] * X[1] * X[1] * X[3];

		*F = 0.0;
		for (j = 0; j < nterms; j++) {
			*F = *F + fabs(f[j]);
		}
	}
	// Nonlinear equations Chemical Equilibrium Application
	// dim = 5 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 16) {
		R = 10.0;
		R5 = 0.193;
		R6 = 0.002597 / sqrt(40.0);
		R7 = 0.003448 / sqrt(40.0);
		R8 = 0.00001799 / 40.0;
		R9 = 0.0002155 / sqrt(40.0);
		R10 = 0.00003846 / 40.0;

		nterms = 5;
		f[0] = X[0] * X[1] + X[0] - 3.0 * X[4];
		f[1] = 2.0 * X[0] * X[1] + X[0] + X[1] * X[2] * X[2] + R8 * X[1] - R * X[4] + 2.0 * R10
				* X[1] * X[1] + R7 * X[1] * X[2] + R9 * X[1] * X[3];
		f[2] = 2.0 * X[1] * X[2] * X[2] + 2.0 * R5 * X[2] * X[2] - 8.0 * X[4] + R6 * X[2] + R7
				* X[1] * X[2];
		f[3] = R9 * X[1] * X[3] + 2.0 * X[3] * X[3] - 4.0 * R * X[4];
		f[4] = X[0] * (1.0 + X[1]) + R10 * X[1] * X[1] + X[1] * X[2] * X[2] + R8 * X[1] + R5 * X[2]
				* X[2] + X[3] * X[3] - 1.0 + R6 * X[2] + R7 * X[1] * X[2] + R9 * X[1] * X[3];

		*F = 0.0;
		for (j = 0; j < nterms; j++) {
			*F = *F + fabs(f[j]);
		}
	}
	// Nonlinear equations Kinematic Application
	// dim = 8 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 17) {
		nterms = 8;

		for (j = 0; j < 4; j++) {
			f[j] = X[j] * X[j] + X[j + 1] * X[j + 1] - 1;
		}

		for (j = 0; j < 4; j++) {
			f[4 + j] = a[0][j] * X[0] * X[2] + a[1][j] * X[0] * X[3] + a[2][j] * X[1] * X[2]
					+ a[3][j] * X[1] * X[3] + a[4][j] * X[1] * X[6] + a[5][j] * X[4] * X[7]
					+ a[6][j] * X[5] * X[6] + a[7][j] * X[5] * X[7] + a[8][j] * X[0] + a[9][j]
					* X[1] + a[10][j] * X[2] + a[11][j] * X[3] + a[12][j] * X[4] + a[13][j] * X[5]
					+ a[14][j] * X[6] + a[15][j] * X[7] + a[16][j];
		}

		*F = 0.0;
		for (j = 0; j < nterms; j++) {
			*F = *F + fabs(f[j]);
		}
	}
	// Nonlinear equations Combustion Application
	// dim = 10 [-10. 10]
	//-------------------
	else if (funcid_.ipro == 18) {

		nterms = 10;
		f[0] = X[1] + 2.0 * X[5] + X[8] + 2.0 * X[9] - 10.0E-5;
		f[1] = X[2] + X[7] - 3.0 * 10.0E-5;
		f[2] = X[0] + X[2] + 2.0 * X[4] + 2.0 * X[7] + X[8] + X[9] - 5.0 * 10.0E-5;
		f[3] = X[3] + 2.0 * X[6] - 10.0E-5;
		f[4] = 0.5140437 * 10.0e-7 * X[4] - X[0] * X[0];
		f[5] = 0.1006932 * 10.0e-6 * X[5] - 2.0 * X[1] * X[1];
		f[6] = 0.7816278 * 10.0e-15 * X[6] - X[3] * X[3];
		f[7] = 0.1496236 * 10.0e-6 * X[7] - X[0] * X[2];
		f[8] = 0.6194411 * 10.0e-7 * X[8] - X[0] * X[1];
		f[9] = 0.2089296 * 10.0e-14 * X[9] - X[0] * X[1] * X[1];

		*F = 0.0;
		for (j = 0; j < nterms; j++) {
			*F = *F + fabs(f[j]);
		}
	}
	// Nonlinear equations Economics Modelling Application
	// dim = arbitrary (tested for dim = 20)
	//-------------------
	else if (funcid_.ipro == 19) {
		double tmp;
		nterms = N;

		*F = 0.0;
		for (j = 0; j < nterms - 1; j++) {
			sum = 0.0;
			for (i = 0; i < nterms - j - 1; i++) {
				sum = sum + X[i] * X[i + j];
			}
			tmp = (X[j] + sum) * X[nterms - 1];
			f[j] = fabs(tmp);
			*F = *F + fabs(tmp);
		}

		sum = 0.0;
		for (l = 0; l < nterms - 1; l++)
			sum = sum + X[l];

		f[nterms - 1] = fabs(sum + 1.0);

		*F = *F + fabs(sum + 1);
	}
	// Shifted Sphere problem 
	// [-100, 100] min: -450
	//-------------------
	else if (funcid_.ipro == 20) {
		func(0, N, X, F);
	}
	// Shifted Rosenbrock problem 
	// [-100, 100] min: 390
	//-------------------
	else if (funcid_.ipro == 21) {
		func(1, N, X, F);
	}
	// Shifted Rastrigin problem 
	// [-5, 5] min: -330 
	//-------------------
	else if (funcid_.ipro == 22) {
		func(2, N, X, F);
	}
	// Shifted Griewank problem 
	// [-600, 600] min: -180
	//-------------------
	else if (funcid_.ipro == 23) {
		func(3, N, X, F);
	}
	// Shifted Ackley problem 
	// [-32, 32] min: -140
	//------------------- 
	else if (funcid_.ipro == 24) {
		func(4, N, X, F);
	}
	// Shifted Schwefel problem 
	// [-100, 100] min: -450
	//-------------------
	else if (funcid_.ipro == 25) {
		func(5, N, X, F);
	}
	// Bird test function problem 
	// [-50, 50] min: -106.765
	//-------------------
	else if (funcid_.ipro == 26) {
		*F = sin(X[0]) * exp((1.0 - cos(X[1])) * (1.0 - cos(X[1]))) + cos(X[1]) * exp((1.0 - sin(
				X[0])) * (1.0 - sin(X[0]))) + (X[0] - X[1]) * (X[0] - X[1]);
	}
	// Bohachevsky test function problem 
	// [-50, 50] min: 0
	//-------------------
	else if (funcid_.ipro == 27) {
		*F = X[0] * X[0] + 2.0 * X[1] * X[1] - 0.3 * cos(3.0 * pi * X[0]) - 0.4 * cos(4.0 * pi
				* X[1]) + 0.7;
	}
	// Carrom table test function problem 
	// [-100, 100] min: 0
	//-------------------
	else if (funcid_.ipro == 28) {
		double expo = 1.0 - (sqrt(X[0] * X[0] + X[1] * X[1]) / (pi));
		*F = -(1.0 / 30.0) * pow((cos(X[0]) * cos(X[1]) * exp(expo)), 2);
	}
	//Guillin hills test function problem 
	// [0, 1] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 29) {
		double sum = 0.0;
		for (i = 0; i < N; i++) {
			sum = sum + (2.0 * (X[i] + 9.0)) / (X[i] + 10) * sin((pi) / (1.0 - X[i] + 1.0 / 10.0));
		}
		*F = 3.0 + sum;
	}
	// Holder test function problem 
	// [-100, 100] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 30) {
		double sum = 0.0, prod = 1.0;
		for (i = 0; i < N; i++) {
			sum = sum + X[i] * X[i];
		}
		for (i = 0; i < N; i++) {
			prod = prod * (-cos(X[i]));
		}

		*F = prod * exp(1.0 - (sqrt(sum)) / (pi));
	}
	// Lagermann test function problem 
	// [0, 100] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 31) {
		double aa[5] = { 3, 5, 2, 1, 7 };
		double cc[5] = { 1, 2, 5, 2, 3 };
		double ss[5], ll[5];

		for (k = 0; k < 5; k++) {
			ss[k] = 0.0;
			ll[k] = 0.0;
			for (i = 0; i < N; i++) {
				ss[k] += -(X[i] - aa[k]) * (X[i] - aa[k]) / pi;
				ll[k] += pi * (X[i] - aa[k]) * (X[i] - aa[k]);
			}
		}
		*F = 0.0;
		for (k = 0; k < 5; k++) {

			*F = *F + cc[k] * exp(ss[k]) * cos(ll[k]);
		}

	}
	// Lennard-Jones Energy
	else if (funcid_.ipro == 32) {
		*F = lj(X, N);
	}

}
//=========================================================================================================
//=========================================================================================================

//----------------------------
// Bounds for Sample objective functions
//----------------------------
void Bounds_F(double *xl, double *xr, int N) {
	/*
	 Input:
	 PROBLEM_par 	prob    - A structure that contains test problem depended parameters
	 funcid_.ipro is the identification number for each test
	 problem
	 Output:
	 double 		*xl 	-  Lower bound for the problem
	 double 		*xr 	-  Upper bound for the problem

	 */
	int i;
	// SPHERE, X=[0,...,0], F=0, [-100,100]
	//--------------------------------------
	if (funcid_.ipro == 0) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}

	// ROSENBROCK, X=[1,...,1], F=0, [-30,30]
	//----------------------------------------
	else if (funcid_.ipro == 1) {
		for (i = 0; i < N; i++) {
			xl[i] = -30.0;
			xr[i] = 30.0;
		}
	}

	// RASTRIGIN, X=[0,...,0], F=0, [-5.12,5.12]
	//-------------------------------------------
	else if (funcid_.ipro == 2) {
		for (i = 0; i < N; i++) {
			xl[i] = -5.12;
			xr[i] = 5.12;
		}
	}

	// GRIEWANK, X=[0,...,0], F=0, [-600,600]
	//----------------------------------------
	else if (funcid_.ipro == 3) {
		for (i = 0; i < N; i++) {
			xl[i] = -600.0;
			xr[i] = 600.0;
		}
	}

	// ACKLEY, X=[0,...,0], F=0, [-20,30]
	//------------------------------------
	else if (funcid_.ipro == 4) {
		for (i = 0; i < N; i++) {
			xl[i] = -20.0;
			xr[i] = 30.0;
		}
	}

	// FREUDENSTEIN-ROTH, X=[5,4], F=0
	//---------------------------------
	else if (funcid_.ipro == 5) {
		for (i = 0; i < N; i++) {
			xl[i] = -5.0;
			xr[i] = 4.0;
		}
	}

	// LEVY No.3, 18 global minimizers with F=-176.542
	//-------------------------------------------------
	else if (funcid_.ipro == 6) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}

	// LEVY, X=[1,...,1], F=0, DIM=3,4,5,8,10
	//----------------------------------------
	else if (funcid_.ipro == 7) {
		for (i = 0; i < N; i++) {
		}
	}

	// POWELL BADLY SCALED, X=[1.098*10^(-5),9.106], F=0
	//---------------------------------------------------
	else if (funcid_.ipro == 8) {
		for (i = 0; i < N; i++) {
		}
	}

	// WOOD, X=[1,1,1,1], F=0
	//------------------------
	else if (funcid_.ipro == 9) {
		for (i = 0; i < N; i++) {
		}
	}

	// TRIGONOMETRIC, F=0
	//--------------------
	else if (funcid_.ipro == 10) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}

	// SCHAFFER F6, X=[0,...,0], F=0, [-100,100]
	//-------------------------------------------
	else if (funcid_.ipro == 11) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}

	// SCHWEFEL'S DOUBLE SUM, X=[0,...,0], F=0, [-65.536, 65.536]
	//------------------------------------------------------------
	else if (funcid_.ipro == 12) {
		for (i = 0; i < N; i++) {
			xl[i] = -65.536;
			xr[i] = 65.536;
		}
	}

	// SCHWEFEL FUNCTION
	//-------------------
	else if (funcid_.ipro == 13) {
		for (i = 0; i < N; i++) {
			xl[i] = -500.0;
			xr[i] = 500.0;
		}
	}
	// Nonlinear equations Interval Arithmetic Benchmark
	// dim = 10 [-2, 2]
	//-------------------
	else if (funcid_.ipro == 14) {
		for (i = 0; i < N; i++) {
			xl[i] = -2.0;
			xr[i] = 2.0;
		}
	}
	// Nonlinear equations Neurophysiology Application Benchmark
	// dim = 6 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 15) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}
	// Nonlinear equations Chemical Equilibrium Application
	// dim = 5 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 16) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}
	// Nonlinear equations Kinematic Application
	// dim = 8 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 17) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}
	// Nonlinear equations Combustion Application
	// dim = 10 [-10. 10]
	//-------------------
	else if (funcid_.ipro == 18) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}
	// Nonlinear equations Economics Modelling Application
	// dim = arbitrary (tested for dim = 20)
	//-------------------
	else if (funcid_.ipro == 19) {
		for (i = 0; i < N; i++) {
			xl[i] = -10.0;
			xr[i] = 10.0;
		}
	}

	else if (funcid_.ipro == 20) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}
	// Shifted Rosenbrock problem 
	// [-100, 100] min: 390
	//-------------------
	else if (funcid_.ipro == 21) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}
	// Shifted Rastrigin problem 
	// [-5, 5] min: -330 
	//-------------------
	else if (funcid_.ipro == 22) {
		for (i = 0; i < N; i++) {
			xl[i] = -5.0;
			xr[i] = 5.0;
		}
	}
	// Shifted Griewank problem 
	// [-600, 600] min: -180
	//-------------------
	else if (funcid_.ipro == 23) {
		for (i = 0; i < N; i++) {
			xl[i] = -600.0;
			xr[i] = 600.0;
		}
	}
	// Shifted Ackley problem 
	// [-32, 32] min: -140
	//------------------- 
	else if (funcid_.ipro == 24) {
		for (i = 0; i < N; i++) {
			xl[i] = -32.0;
			xr[i] = 32.0;
		}
	}
	// Shifted Schwefel problem 
	// [-100, 100] min: -450
	//-------------------
	else if (funcid_.ipro == 25) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}
	// Bird problem 
	// [-50, 50] min: xxx
	//-------------------
	else if (funcid_.ipro == 26) {
		for (i = 0; i < N; i++) {
			xl[i] = -50.0;
			xr[i] = 50.0;
		}
	}
	// Bohachescky problem 
	// [-10, 10] min: 0
	//-------------------
	else if (funcid_.ipro == 27) {
		for (i = 0; i < N; i++) {
			xl[i] = -50.0;
			xr[i] = 50.0;
		}
	}
	// Carrom table  problem 
	// [-100, 100] min: 0.24632
	//-------------------
	else if (funcid_.ipro == 28) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}
	//Guillin hills test function problem 
	// [0, 1] min: 0
	//-------------------
	else if (funcid_.ipro == 29) {
		for (i = 0; i < N; i++) {
			xl[i] = 0.0;
			xr[i] = 1.0;
		}
	}
	// Holder test function problem 
	// [-100, 100] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 30) {
		for (i = 0; i < N; i++) {
			xl[i] = -100.0;
			xr[i] = 100.0;
		}
	}
	// Lagermann test function problem 
	// [0, 1] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 31) {
		for (i = 0; i < N; i++) {
			xl[i] = 0.0;
			xr[i] = 100.0;
		}
	}
	// Lennard-Jones
	// [-2.2, 2.2] min: xxxx
	//-------------------
	else if (funcid_.ipro == 32) {
		for (i = 0; i < N; i++) {
			xl[i] = -2.0;
			xr[i] = 2.0;
		}
	}
}

void Minimum_X(double *sol, int N) {
	int i;
	// SPHERE, X=[0,...,0], F=0, [-100,100]
	//--------------------------------------
	if (funcid_.ipro == 0) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;
	}

	// ROSENBROCK, X=[1,...,1], F=0, [-30,30]
	//----------------------------------------
	else if (funcid_.ipro == 1) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;
	}

	// RASTRIGIN, X=[0,...,0], F=0, [-5.12,5.12]
	//-------------------------------------------
	else if (funcid_.ipro == 2) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;
	}

	// GRIEWANK, X=[0,...,0], F=0, [-600,600]
	//----------------------------------------
	else if (funcid_.ipro == 3) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;
	}

	// ACKLEY, X=[0,...,0], F=0, [-20,30]
	//------------------------------------
	else if (funcid_.ipro == 4) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;
	}

	// FREUDENSTEIN-ROTH, X=[5,4], F=0
	//---------------------------------
	else if (funcid_.ipro == 5) {
		sol[0] = 5.0;
		sol[1] = 4.0;
	}

	// LEVY No.3, 18 global minimizers with F=-176.542
	//-------------------------------------------------
	else if (funcid_.ipro == 6) {

	}

	// LEVY, X=[1,...,1], F=0, DIM=3,4,5,8,10
	//----------------------------------------
	else if (funcid_.ipro == 7) {

	}

	// POWELL BADLY SCALED, X=[1.098*10^(-5),9.106], F=0
	//---------------------------------------------------
	else if (funcid_.ipro == 8) {

	}

	// WOOD, X=[1,1,1,1], F=0
	//------------------------
	else if (funcid_.ipro == 9) {

	}

	// TRIGONOMETRIC, F=0
	//--------------------
	else if (funcid_.ipro == 10) {

	}

	// SCHAFFER F6, X=[0,...,0], F=0, [-100,100]
	//-------------------------------------------
	else if (funcid_.ipro == 11) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;

	}

	// SCHWEFEL'S DOUBLE SUM, X=[0,...,0], F=0, [-65.536, 65.536]
	//------------------------------------------------------------
	else if (funcid_.ipro == 12) {
		for (i = 0; i < N; i++)
			sol[i] = 0.0;

	}

	// SCHWEFEL FUNCTION
	//-------------------
	else if (funcid_.ipro == 13) {
		for (i = 0; i < N; i++)
			sol[i] = -420.9687;

	}
	// Nonlinear equations Interval Arithmetic Benchmark
	// dim = 10 [-2, 2]
	//-------------------
	else if (funcid_.ipro == 14) {

	}
	// Nonlinear equations Neurophysiology Application Benchmark
	// dim = 6 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 15) {

	}
	// Nonlinear equations Chemical Equilibrium Application
	// dim = 5 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 16) {

	}
	// Nonlinear equations Kinematic Application
	// dim = 8 [-10, 10]
	//-------------------
	else if (funcid_.ipro == 17) {

	}
	// Nonlinear equations Combustion Application
	// dim = 10 [-10. 10]
	//-------------------
	else if (funcid_.ipro == 18) {

	}
	// Nonlinear equations Economics Modelling Application
	// dim = arbitrary (tested for dim = 20)
	//-------------------
	else if (funcid_.ipro == 19) {

	}
	// Shifted Sphere problem 
	// [-100, 100] min: 
	else if (funcid_.ipro == 20) {

	}
	// Shifted Rosenbrock problem 
	// [-100, 100] min: 390
	//-------------------
	else if (funcid_.ipro == 21) {

	}
	// Shifted Rastrigin problem 
	// [-5, 5] min: -330 
	//-------------------
	else if (funcid_.ipro == 22) {

	}
	// Shifted Griewank problem 
	// [-600, 600] min: -180
	//-------------------
	else if (funcid_.ipro == 23) {

	}
	// Shifted Ackley problem 
	// [-32, 32] min: -140
	//------------------- 
	else if (funcid_.ipro == 24) {

	}
	// Shifted Schwefel problem 
	// [-100, 100] min: -450
	//-------------------
	else if (funcid_.ipro == 25) {

	}
	// Bird problem 
	// [-50, 50] min: xxx
	//-------------------
	else if (funcid_.ipro == 26) {

	}
	// Bohachescky problem 
	// [-10, 10] min: 0
	//-------------------
	else if (funcid_.ipro == 27) {

	}
	// Carrom table  problem 
	// [-10, 10] min: xxx
	//-------------------
	else if (funcid_.ipro == 28) {

	}
	//Guillin hills test function problem 
	// [0, 1] min: 0
	//-------------------
	else if (funcid_.ipro == 29) {

	}
	// Holder test function problem 
	// [-100, 100] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 30) {

	}
	// Lagermann test function problem 
	// [0, 1] min: scaled with dimension
	//-------------------
	else if (funcid_.ipro == 31) {

	} else if (funcid_.ipro == 26) {

	}

}

void func(int func_number, int dim, double* x, double *fx) {

	pi = acos(-1.0);
	e = exp(1.0);
	double tmp[dim];

	if (dim > 1000)
		dim = 1000;

	switch (func_number) {
	case 0:
		*fx = Shifted_Sphere(dim, x);
		copy_vector(dim, sphere, tmp);
		break;
	case 1:
		*fx = Shifted_Rosenbrock(dim, x);
		copy_vector(dim, rosenbrock, tmp);
		break;
	case 2:
		*fx = Shifted_Rastrigin(dim, x);
		copy_vector(dim, rastrigin, tmp);
		break;
	case 3:
		*fx = Shifted_Griewank(dim, x);
		copy_vector(dim, griewank, tmp);
		break;
	case 4:
		*fx = Shifted_Ackley(dim, x);
		copy_vector(dim, ackley, tmp);
		break;
	case 5:
		*fx = Schwefel_Problem(dim, x);
		copy_vector(dim, schwefel, tmp);
		break;
	default:
		printf("Error : Function number out of range\n");
		exit(0);
		break;
	}

}

double Shifted_Sphere(int dim, double* x) {
	int i;
	double z;
	double F = 0;
	for (i = 0; i < dim; i++) {
		z = x[i] - sphere[i];
		F += z * z;
	}
	return F + f_bias[0];
}

double Schwefel_Problem(int dim, double* x) {
	int i;
	double z;
	double F = abss(x[0]);
	for (i = 1; i < dim; i++) {
		z = x[i] - schwefel[i];
		F = max(F , abss(z));
	}
	return F + f_bias[1];
}

double Shifted_Rosenbrock(int dim, double* x) {
	int i;
	double z[dim];
	double F = 0;

	for (i = 0; i < dim; i++)
		z[i] = x[i] - rosenbrock[i] + 1;

	for (i = 0; i < dim - 1; i++) {
		F = F + 100 * (pow((pow(z[i], 2) - z[i + 1]), 2)) + pow((z[i] - 1), 2);
	}
	return F + f_bias[2];
}

double Shifted_Rastrigin(int dim, double* x) {
	int i;
	double z;
	double F = 0;
	for (i = 0; i < dim; i++) {
		z = x[i] - rastrigin[i];
		F = F + (pow(z, 2) - 10 * cos(2 * pi * z) + 10);
	}
	return F + f_bias[3];
}

double Shifted_Griewank(int dim, double* x) {
	int i;
	double z;
	double F1 = 0;
	double F2 = 1;
	for (i = 0; i < dim; i++) {
		z = x[i] - griewank[i];
		F1 = F1 + (pow(z, 2) / 4000);
		F2 = F2 * (cos(z / sqrt(i + 1)));

	}
	return (F1 - F2 + 1 + f_bias[4]);
}

double Shifted_Ackley(int dim, double* x) {
	int i;
	double z;
	double Sum1 = 0;
	double Sum2 = 0;
	double F = 0;
	for (i = 0; i < dim; i++) {
		z = x[i] - ackley[i];
		Sum1 = Sum1 + pow(z, 2);
		Sum2 = Sum2 + cos(2 * pi * z);
	}
	F = -20 * exp(-0.2 * sqrt(Sum1 / dim)) - exp(Sum2 / dim) + 20 + e + f_bias[5];

	return F;
}

double lj(double p[], int n) {
	int np, i, j, i3, j3, np1;
	double v, dx, dy, dz, rij;
	double xi, yi, zi;
	double xj, yj, zj;

	np = n / 3;

	v = 0.0;
	np1 = np - 1;
	for (i = 0; i < np1; ++i) {
		i3 = 3 * i;
		xi = p[i3];
		yi = p[i3 + 1];
		zi = p[i3 + 2];
		for (j = i + 1; j < np; ++j) {
			j3 = 3 * j;
			xj = p[j3];
			yj = p[j3 + 1];
			zj = p[j3 + 2];
			dx = xi - xj;
			dy = yi - yj;
			dz = zi - zj;
			rij = sqrt(dx * dx + dy * dy + dz * dz);
			v += ljpot(rij);
		}
	}
	return v;
}

#define SIGMA    1.0
#define EPSILON4 4.0
double ljpot(double r) {
	double sr, sr3, sr6, sr12;

	sr = SIGMA / r;
	sr3 = sr * sr * sr;
	sr6 = sr3 * sr3;
	sr12 = sr6 * sr6;
	return EPSILON4 * (sr12 - sr6);
}

