/*
 =========================================================================================================
 C. Voglis, Department of Computer Science,
 Univeristy of Ioannina, Greece

 File: 				bfgs.c
 Usage: 			A C port of the BFGSMV method implemented in Merlin 3.1
 Last Modified: 	21/08/2013
 =========================================================================================================
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#define TRUE_ 1
#define FALSE_ 0

extern double funmin_(double *, int *);
extern void Objective_F(double *, int, double *);
extern void Objective_G(double *, int, double *);

extern void ondlga_(double *X, int *N, double *XL, double *XU, double *FEPS, int *IORD,
		int *IPRINT, double *G, int *NOC, int *IERR);

double max(double a, double b) {
	if (a > b)
		return a;
	else
		return b;
}
double min(double a, double b) {
	if (a < b)
		return a;
	else
		return b;
}
double d_sign(double *a, double *b) {
	if (*b > 0)
		if (*a > 0)
			return *a;
		else
			return -(*a);
	else if (*a > 0)
		return -(*a);
	else
		return (*a);
}

int c__0 = 0;
int c__1 = 1;
int c__9 = 9;

/* Subroutine */int bfgsmv_(double *h__, double *point, double * grad, int *nod, double *val,
		int *nomaxc, double * comacc, double *gtol, int *itmax, double *xtol, int * lsmeth,
		double *rho, double *sigma, int *lsiter, double *ftol, int *info, int *iter, int *nocex,
		int *nogex, int *iprint, int *ixat, int *marg, double *xll, double *xrl, int *igradu,
		int *ihessu, double *xs, double *gn, double *y, double *gs, int analytic, int iord) {
	/* System generated locals */
	int h_dim1, h_offset, i__1, i__2;

	/* Builtin functions */
	double sqrt(double);

	/* Local variables */
	int i__, j, m, n1, n2;
	double aa = 0.0;
	int mm = 0;
	double tt = 0.0;
	double dtd = 0.0;
	int nof = 0;
	double yts, yty;
	extern /* Subroutine */int qnls_(double *, double *, int *, int *, double *, double *, int *,
			double *, double *, int *, double *, double *, int *, int *, int *, int *, double *,
			double *, double *, double *, int, int);
	double alpha = 0.0;
	int minfo = 0;
	extern /* Subroutine */int llupd_(double *, double *, double *, int *, int *), lmult_(double *,
			double *, double *, int *, int *), granal_(int *, double *, double *), freeva_(int *,
			int *, int *, int *, int *);
	double oldval = 0.0;
	extern int igconv_(double *, double *, int *, double *, double *), ifconv_(double *, double *,
			double *), itconv_(double *), ixconv_(double *, double *, int *, double *);
	extern /* Subroutine */int llsolv_(double *, double *, double *, int *, int *), ltmult_(
			double *, double *, double *, int *, int *), vvmult_(double *, double *, double *,
			int *);

	int IPRINT, NOC, IERR;

	int IORD = iord;
	int jj;

	i__ = 0;
	j = 0;
	m = 0;
	n1 = 0;
	n2 = 0;

	/*  Number of iterations performed so far. */
	/* Parameter adjustments */
	--gs;
	--y;
	--gn;
	--xs;
	--xrl;
	--xll;
	--marg;
	--ixat;
	--grad;
	--point;

	h_dim1 = *nod;
	h_offset = 1 + h_dim1;
	h__ -= h_offset;

	/* Function Body */
	*iter = 0;

	/*  Number of function calls so far. */
	*nocex = 0;

	/*  Number of (analytic) gradient calls so far. */
	*nogex = 0;

	/*  Did we reach our target value ? */
	if (itconv_(val) != 0) {
		*info = 1;
		return 0;
	}

	/*  Count non-fixed variables. M will be used as the dimensionality */
	/*  of the objective function in all subsequent matrix/vector operations. */
	freeva_(&m, &ixat[1], &marg[1], nod, &nof);
	if (m == 0) {
		*info = 8;
		return 0;
	}

	/*  Calculate derivatives at the starting point. */
	if (analytic == 1) {
		Objective_G(&point[1], m, &grad[1]);
		++(*nogex);
	} else {

		for (jj = 1; jj <= m; jj++)
		{
			if (point[jj] <= xll[jj])
				point[jj] = xll[jj] + 2.0 * (*ftol);
			if (point[jj] >= xrl[jj])
				point[jj] = xrl[jj] - 2.0 * (*ftol);
		}

		IPRINT = 0;
		ondlga_(&point[1], &m, &xll[1], &xrl[1], ftol, &IORD, &IPRINT, &grad[1], &NOC, &IERR);
		if (IERR != 0) {
			printf("Abort on numeric derivatives(1) \n");
			exit(0);
		}
		*nocex = *nocex + NOC;
	}

	/*  Copy derivatives to an array of M elements. */
	mm = 0;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		if (ixat[i__] != 0) {
			++mm;
			gs[mm] = grad[i__];
		}
		/* L17: */
	}

	/*  Check the gradient criterion for the initial point. */
	/*  The numerical differentiation routine DERMER is expected to */
	/*  return GRAD(I) = 0 for all fixed variables. Otherwise the */
	/*  following test will fail, since the whole array GRAD is examined. */
	if (igconv_(&grad[1], &point[1], nod, val, gtol) != 0) {
		*info = 2;
		return 0;
	}

	/*  Initialize Hessian. (Upper triangular part only.) */
	if (*ihessu == 0) {
		i__1 = m;
		for (j = 1; j <= i__1; ++j) {
			h__[j + j * h_dim1] = 1.f;
			i__2 = j - 1;
			for (i__ = 1; i__ <= i__2; ++i__) {
				/* L20: */
				h__[i__ + j * h_dim1] = 0.f;
			}
			/* L10: */
		}
	}
	/*  --------------------------------------------------------------------- */
	/*  The following loop performs BFGS iterations. */
	/*  --------------------------------------------------------------------- */
	L1000: if (*itmax != -1 && *iter >= *itmax) {
		*info = 7;
		return 0;
	}
	if (*nocex >= *nomaxc) {
		*info = 3;
		return 0;
	}
	++(*iter);

	/*  Solve H * s = -g for the search direction XS. The minus sign will */
	/*  be added later. */
	llsolv_(&h__[h_offset], &xs[1], &gs[1], &m, nod);

	/*  Save current point and value for the convergence tests. */
	oldval = *val;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L99: */
		gn[i__] = point[i__];
	}

	/*  Prepare a vector of length NOD with the proper line search */
	/*  direction. */
	mm = 0;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		if (ixat[i__] != 0) {
			++mm;
			gs[i__] = -xs[mm];
		} else {
			gs[i__] = 0.f;
		}
		/* L71: */
	}

	/*  Initial estimate for the line search. */
	alpha = 1.f;

	/*  Perform the line search. Here, XS and Y are used as work arrays. */
	qnls_(&gs[1], &grad[1], &n1, &n2, &alpha, comacc, lsmeth, rho, sigma, lsiter, &point[1], val,
			nod, &minfo, &ixat[1], &marg[1], &xll[1], &xrl[1], &xs[1], &y[1], analytic, IORD);
	*nocex += n1;
	*nogex += n2;

	/*  Any progress during the line search ? */
	if (alpha == 0.f) {
		*info = 9;
		return 0;
	}

	/*  Did we reach our target value ? */
	if (itconv_(val) != 0) {
		*info = 1;
		return 0;
	}

	/*  Check for X-convergence. */
	if (ixconv_(&point[1], &gn[1], nod, xtol) != 0) {
		*info = 4;
		return 0;
	}

	/*  Check for F-convergence. */
	if (ifconv_(val, &oldval, ftol) != 0) {
		*info = 5;
		return 0;
	}

	/*  Calculate derivatives at the point the line search stopped. */
	if (analytic == 1) {
		Objective_G(&point[1], m, &gn[1]);
		++(*nogex);
	} else {
		for (jj = 1; jj <= m; jj++) {
			if (point[jj] <= xll[jj])
				point[jj] = xll[jj] + 2.0 * (*ftol);
			if (point[jj] >= xrl[jj])
				point[jj] = xrl[jj] - 2.0 * (*ftol);
		}

		ondlga_(&point[1], &m, &xll[1], &xrl[1], ftol, &IORD, &IPRINT, &gn[1], &NOC, &IERR);
		if (IERR != 0) {
			printf("Abort on numeric derivatives (2)\n");
			exit(0);
		}

		*nocex = *nocex + NOC;
	}



	/*  Check the gradient criterion. */
	if (igconv_(&gn[1], &point[1], nod, val, gtol) != 0) {
		*info = 2;
		return 0;
	}

	/*  Calculate the difference in the gradient: Y = g[k+1]-g[k] */
	/*  and in the variables: XS = x[k+1] - x[k] */
	/*  Update derivatives in GRAD. */
	mm = 0;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		if (ixat[i__] != 0) {
			++mm;
			xs[mm] = alpha * gs[i__];
			y[mm] = gn[i__] - grad[i__];
			gs[mm] = gn[i__];
		}
		grad[i__] = gn[i__];
		/* L13: */
	}

	/*  Calculate vector inner products needed to determine whether the */
	/*  Hessian can be updated. */
	vvmult_(&y[1], &y[1], &yty, &m);
	vvmult_(&xs[1], &xs[1], &dtd, &m);
	vvmult_(&y[1], &xs[1], &yts, &m);

	/*  Can we update the Hessian ? */
	if (yts > sqrt(*comacc * dtd * yty)) {

		/*  XS = L[t] * s */
		ltmult_(&h__[h_offset], &xs[1], &xs[1], &m, nod);

		/*  TT = s[t] * H * s */
		vvmult_(&xs[1], &xs[1], &tt, &m);
		if (tt == 0.f) {
			*info = 6;
			return 0;
		}

		/*  GN = H * s */
		lmult_(&h__[h_offset], &xs[1], &gn[1], &m, nod);
		aa = sqrt(yts / tt);
		i__1 = m;
		for (i__ = 1; i__ <= i__1; ++i__) {
			/* L57: */
			gn[i__] = aa * (y[i__] - aa * gn[i__]) / yts;
		}

		/*  Update Choleski factors. */
		llupd_(&h__[h_offset], &gn[1], &xs[1], &m, nod);
	}
	goto L1000;
	/*  --------------------------------------------------------------------- */
	/*  End of BFGS iterations. */
	/*  --------------------------------------------------------------------- */

} /* bfgsmv_ */

/* Subroutine */int freeva_(int *nfree, int *ixat, int *marg, int *nod, int *nof) {
	/* System generated locals */
	int i__1;

	/* Local variables */
	int i__;

	/* Parameter adjustments */
	--marg;
	--ixat;

	/* Function Body */
	*nfree = 0;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L10: */
		if (ixat[i__] != 0) {
			++(*nfree);
		}
	}

	return 0;
} /* freeva_ */

/* Subroutine */int llsolv_(double *a, double *x, double *b, int *n, int *ld) {
	/* System generated locals */
	int a_dim1, a_offset, i__1, i__2;

	/* Local variables */
	int i__, j;
	double sum;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--b;
	--x;
	a_dim1 = *ld;
	a_offset = 1 + a_dim1;
	a -= a_offset;

	/* Function Body */
	i__1 = *n;
	for (i__ = 1; i__ <= i__1; ++i__) {
		sum = 0.f;
		i__2 = i__ - 1;
		for (j = 1; j <= i__2; ++j) {
			/* L20: */
			sum += a[j + i__ * a_dim1] * x[j];
		}
		x[i__] = (b[i__] - sum) / a[i__ + i__ * a_dim1];
		/* L10: */
	}

	for (i__ = *n; i__ >= 1; --i__) {
		sum = 0.f;
		i__1 = *n;
		for (j = i__ + 1; j <= i__1; ++j) {
			/* L40: */
			sum += a[i__ + j * a_dim1] * x[j];
		}
		x[i__] = (x[i__] - sum) / a[i__ + i__ * a_dim1];
		/* L30: */
	}

	return 0;
} /* llsolv_ */

/* Subroutine */int qnls_(double *dir, double *grad, int *nocall, int *nog, double *alpha,
		double *comacc, int *lsmeth, double *rho, double *sigma, int *lsiter, double * point,
		double *val, int *nod, int *info, int *ixat, int *marg, double *xll, double *xrl,
		double *work, double *work2, int analytic, int iord) {
	/* System generated locals */
	int i__1;
	double d__1, d__2, d__3, d__4, d__5;

	/* Local variables */
	double g;
	int i__;
	double dd;
	int ma;
	double cp, pp, pp0, ala, one, ten;
	int lim;
	extern /* Subroutine */int lns_(double *, double *, double *, int *, double *, double *,
			double *, int *, double *, double *, double *, double *, double *, double *, int *,
			int *, int *, int *, int *, double *, int *, double *, double *, int *, int, int);
	int isw;
	double ftmp, xmax, ftarg;
	extern /* Subroutine */int meval_(double *, double *, double *, int *, int *, int *, double *,
			double *, double *, int *, double *);
	double xalim, xblim, fxmin, alamin;
	int nfixed;
	extern /* Subroutine */int dirlim_(double *, double *, int *, double *, double *, int *, int *,
			double *, double *);
	double bignum;
	extern /* Subroutine */int vvmult_(double *, double *, double *, int *);

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--work2;
	--work;
	--xrl;
	--xll;
	--marg;
	--ixat;
	--point;
	--grad;
	--dir;

	/* Function Body */
	bignum = 1e300;

	/*  Prepare target value. Take into account the TARGET command. */
	ftarg = -bignum;

	/*  Some constants. */
	one = 1.f;
	ten = 10.f;

	/*  Calculate minimum acceptable ALPHA. Too small ALPHA values would */
	/*  cause X-convergence to be detected in the calling routine. */
	xmax = 0.f;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L40: */
		/* Computing MAX */
		/* Computing MAX */
		d__5 = (d__2 = point[i__], fabs(d__2));
		d__3 = xmax, d__4 = (d__1 = dir[i__], fabs(d__1)) / max(d__5, one);
		xmax = max(d__3, d__4);
	}

	/*  XMAX cannot be zero, since convergence would have been detected */
	/*  by the calling routine. The following division is safe. */
	/* Computing MIN */
	d__1 = *comacc / xmax;
	alamin = min(d__1, ten);

	/*  Consider fixed the parameters that result in very small ALPHA. */
	ala = alamin * 10.f;
	pp0 = ala * 10.f;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		dd = dir[i__];
		if (dd != 0.) {
			ma = marg[i__];
			pp = pp0;
			cp = point[i__];
			if (dd > 0.f) {
				if (ma == 1 || ma == 2) {
					pp = (xrl[i__] - cp) / dd;
				}
			} else {
				if (ma == -1 || ma == 2) {
					pp = (xll[i__] - cp) / dd;
				}
			}
			if (pp < ala) {
				dir[i__] = 0.f;
			}
		}
		/* L11: */
	}

	/*  Count zero directions. */
	nfixed = 0;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L1: */
		if (dir[i__] == 0.f) {
			++nfixed;
		}
	}

	/*  If all parameters are fixed then return. */
	/*  Note that at this point all parameters cannot be fixed due to FIX */
	/*  statements, since this would have been detected by the calling */
	/*  routine. If the direction vector DIR is zero, this is due to the */
	/*  margins. The calling routine may want to attempt a steepest descent */
	/*  (or some other) restart. */
	if (nfixed == *nod) {
		*alpha = 0.;
		*info = -1;
		return 0;
	}

	/*  Calculate margins XALIM, XBLIM along the direction of search. */
	dirlim_(&point[1], &dir[1], &marg[1], &xll[1], &xrl[1], nod, &lim, &xalim, &xblim);

	/*  Setting DIR(I) = 0 for some parameters (due to the existence of */
	/*  margins) may result in a non-descent direction. */
	/*  Warning: The calling method should be prepared to handle negative */
	/*  values for ALPHA. */
	isw = 0;
	vvmult_(&dir[1], &grad[1], &g, nod);
	if (g > 0.f && lim != 0) {
		i__1 = *nod;
		for (i__ = 1; i__ <= i__1; ++i__) {
			/* L88: */
			dir[i__] = -dir[i__];
		}
		xblim = fabs(xalim);
		isw = 1;
	}

	if (xblim == 0.f && lim != 0) {
		*alpha = 0.f;
		*nocall = 0;
		*nog = 0;
	} else {
		if (*alpha < alamin) {
			*alpha = alamin * 1.2f;
		}
		lns_(&point[1], &grad[1], &dir[1], nod, val, rho, sigma, lsiter, &ftarg, alpha, &fxmin,
				comacc, &work[1], &work2[1], lsmeth, nocall, nog, info, &lim, &xblim, &marg[1],
				&xll[1], &xrl[1], &ixat[1], analytic, iord);
	}

	if (isw == 1) {
		i__1 = *nod;
		for (i__ = 1; i__ <= i__1; ++i__) {
			/* L77: */
			dir[i__] = -dir[i__];
		}
		*alpha = -(*alpha);
	}

	/*  Calculate the new point and make sure it lies inside the margins. */
	meval_(&point[1], &dir[1], alpha, nod, &ixat[1], &marg[1], &xll[1], &xrl[1], &point[1], &c__0,
			&ftmp);
	*val = fxmin;

	return 0;
} /* qnls_ */

/* Subroutine */int ltmult_(double *a, double *x, double *b, int *n, int *ld) {
	/* System generated locals */
	int a_dim1, a_offset, i__1, i__2;

	/* Local variables */
	int i__, j;
	double sum;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--b;
	--x;
	a_dim1 = *ld;
	a_offset = 1 + a_dim1;
	a -= a_offset;

	/* Function Body */
	i__1 = *n;
	for (i__ = 1; i__ <= i__1; ++i__) {
		sum = 0.f;
		i__2 = *n;
		for (j = i__; j <= i__2; ++j) {
			/* L20: */
			sum += a[i__ + j * a_dim1] * x[j];
		}
		b[i__] = sum;
		/* L10: */
	}

	return 0;
} /* ltmult_ */

/* Subroutine */int vvmult_(double *x, double *y, double *xy, int *n) {
	/* System generated locals */
	int i__1;

	/* Local variables */
	int i__;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--y;
	--x;

	/* Function Body */
	*xy = 0.f;
	i__1 = *n;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L10: */
		*xy += x[i__] * y[i__];
	}

	return 0;
} /* vvmult_ */

/* Subroutine */int lmult_(double *a, double *x, double *b, int *n, int *ld) {
	/* System generated locals */
	int a_dim1, a_offset, i__1;

	/* Local variables */
	int i__, j;
	double sum;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--b;
	--x;
	a_dim1 = *ld;
	a_offset = 1 + a_dim1;
	a -= a_offset;

	/* Function Body */
	for (i__ = *n; i__ >= 1; --i__) {
		sum = 0.f;
		i__1 = i__;
		for (j = 1; j <= i__1; ++j) {
			/* L20: */
			sum += a[j + i__ * a_dim1] * x[j];
		}
		b[i__] = sum;
		/* L10: */
	}

	return 0;
} /* lmult_ */

/* Subroutine */int llupd_(double *a, double *v, double *u, int *n, int *ld) {
	/* System generated locals */
	int a_dim1, a_offset, i__1;
	double d__1, d__2;

	/* Builtin functions */
	double sqrt(double);

	/* Local variables */
	int i__, j, k;
	double aa, bb;
	extern /* Subroutine */int jrot_(double *, int *, double *, double *, int *, int *);

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--u;
	--v;
	a_dim1 = *ld;
	a_offset = 1 + a_dim1;
	a -= a_offset;

	/* Function Body */
	i__1 = *n;
	for (i__ = 2; i__ <= i__1; ++i__) {
		/* L60: */
		a[i__ + (i__ - 1) * a_dim1] = 0.f;
	}

	/*  Search for non-zero U(I). */
	for (k = *n; k >= 1; --k) {
		/* L10: */
		if (u[k] != 0.f) {
			goto L20;
		}
	}
	k = 1;

	L20: for (i__ = k - 1; i__ >= 1; --i__) {
		aa = u[i__];
		bb = -u[i__ + 1];
		jrot_(&a[a_offset], &i__, &aa, &bb, n, ld);
		if (aa == 0.f) {
			u[i__] = fabs(bb);
		} else {
			/* Computing 2nd power */
			d__1 = aa;
			/* Computing 2nd power */
			d__2 = bb;
			u[i__] = sqrt(d__1 * d__1 + d__2 * d__2);
		}
		/* L30: */
	}

	i__1 = *n;
	for (j = 1; j <= i__1; ++j) {
		/* L40: */
		a[j * a_dim1 + 1] += u[1] * v[j];
	}

	i__1 = k - 1;
	for (i__ = 1; i__ <= i__1; ++i__) {
		aa = a[i__ + i__ * a_dim1];
		bb = -a[i__ + 1 + i__ * a_dim1];
		jrot_(&a[a_offset], &i__, &aa, &bb, n, ld);
		/* L50: */
	}

	return 0;
} /* llupd_ */

/* Subroutine */int dirlim_(double *point, double *dir, int * marg, double *xll, double *xrl,
		int *nod, int *lim, double *xalim, double *xblim) {
	/* System generated locals */
	int i__1;

	/* Local variables */
	int i__, ma, nl, nr;
	double xtlim, bignum;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--xrl;
	--xll;
	--marg;
	--dir;
	--point;

	/* Function Body */
	bignum = 1e300;

	/*  Find left limit along the search direction. */
	nl = 0;
	*xalim = -bignum;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		ma = marg[i__];
		if (dir[i__] == 0.f || ma == 0) {
			goto L10;
		}
		if (dir[i__] > 0.) {
			if (ma == -1 || ma == 2) {
				xtlim = (xll[i__] - point[i__]) / dir[i__];
				++nl;
			} else {
				xtlim = -bignum;
			}
		} else {
			if (ma == 1 || ma == 2) {
				xtlim = (xrl[i__] - point[i__]) / dir[i__];
				++nl;
			} else {
				xtlim = -bignum;
			}
		}
		if (xtlim > *xalim) {
			*xalim = xtlim;
		}
		L10: ;
	}

	/*  Find right limit along the search direction. */
	nr = 0;
	*xblim = bignum;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		ma = marg[i__];
		if (dir[i__] == 0.f || ma == 0) {
			goto L20;
		}
		if (dir[i__] > 0.f) {
			if (ma == 1 || ma == 2) {
				xtlim = (xrl[i__] - point[i__]) / dir[i__];
				++nr;
			} else {
				xtlim = bignum;
			}
		} else {
			if (ma == -1 || ma == 2) {
				xtlim = (xll[i__] - point[i__]) / dir[i__];
				++nr;
			} else {
				xtlim = bignum;
			}
		}
		if (xtlim < *xblim) {
			*xblim = xtlim;
		}
		L20: ;
	}

	/*  LIM indicates whether margins exist along the search direction. */
	if (nl == 0 && nr == 0) {
		*lim = 0;
	} else if (nl == 0) {
		*lim = 1;
	} else if (nr == 0) {
		*lim = -1;
	} else {
		*lim = 2;
	}

	/*  Left margin should always be negative, right margin positive. */
	/*  This is not always true due to small round off errors in the */
	/*  calculation of XALIM and XBLIM. */
	if ((*lim == -1 || *lim == 2) && *xalim > 0.f) {
		*xalim = 0.f;
	}
	if ((*lim == 1 || *lim == 2) && *xblim < 0.f) {
		*xblim = 0.f;
	}

	return 0;
} /* dirlim_ */

/* Subroutine */int lns_(double *point, double *grad, double * dir, int *nod, double *f0,
		double *rho, double *sigma, int *maxit, double *ftarg, double *al, double *fal,
		double *comacc, double *w, double *xtg, int *method, int *nfc, int *ngc, int *info,
		int *lim, double * xlim, int *marg, double *xll, double *xrl, int *ixat, int analytic, int iord) {
	/* System generated locals */
	int i__1;
	double d__1, d__2, d__3, d__4, d__5;

	/* Builtin functions */

	/* Local variables */
	double a, b;
	int i__;
	double g0, t1, t2, t3, t4, t5, t6, aa, bb, fa, ga, fb;
	int nr;
	double aal, gal;
	int ibk;
	int nob;
	double one, ten;
	extern /* Subroutine */int cmin_(double *, double *, double *, double *, double *, double *,
			int *, double *);
	int iter;
	extern /* Subroutine */int qmin_(double *, double *, double *, double *, double *, int *,
			double *);
	double flow, xmin, xmax;
	extern /* Subroutine */int cmin2_(double *, double *, double *, double *, double *, double *,
			double *, int *, double *), keepe_(double *, double *, double *, double *, int *,
			double *, double *, double *, double *);
	int iside;
	extern /* Subroutine */int meval_(double *, double *, double *, int *, int *, int *, double *,
			double *, double *, int *, double *), keept_(double *, double *, double *, double *,
			double *);
	double allow;
	int forceb;
	double alamin;
	extern /* Subroutine */int granal_(int *, double *, double *), vvmult_(double *, double *,
			double *, int *);
	extern void Objective_G(double *, int, double *);

	int  IPRINT, IERR, NOC;
	double ftol;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--ixat;
	--xrl;
	--xll;
	--marg;
	--xtg;
	--w;
	--dir;
	--grad;
	--point;

	/* Function Body */
	one = 1.f;
	ten = 10.f;

	/*  Truncation constants. */
	t1 = .1f;
	t2 = .5f;
	t3 = t1 + 1;
	t4 = 10.f;
	t5 = .1f;
	t6 = .5f;

	/*  FLOW is the lowest function value so far. It need not satisfy */
	/*  the line-search conditions and is returned whenever a normal */
	/*  return is not possible. */
	flow = *f0;
	allow = 0.f;

	/*  Zero counters (function calls, gradient calls). */
	*nfc = 0;
	*ngc = 0;

	/*  Calculate directional derivative at the starting point. */
	vvmult_(&grad[1], &dir[1], &g0, nod);
	if (g0 >= 0.f) {
		/*  This is not a descent direction. */
		*al = allow;
		*fal = flow;
		*info = 2;
		return 0;
	}

	/*  Obtain a lower bound on AL. */
	xmax = 0.f;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* L40: */
		/* Computing MAX */
		/* Computing MAX */
		d__5 = (d__2 = point[i__], fabs(d__2));
		d__3 = xmax, d__4 = (d__1 = dir[i__], fabs(d__1)) / max(d__5, one);
		xmax = max(d__3, d__4);
	}
	/* Computing MIN */
	d__1 = *comacc / xmax;
	alamin = min(d__1, ten);

	/*  Initial values for the left bracket. */
	a = 0.f;
	fa = *f0;
	ga = g0;

	/*  NOB = .TRUE. indicates that a right bracket has not been found. */
	nob = TRUE_;

	/*  IBK is the number of backtracks towards the left bracket. */
	/*  Note that the left bracket always satisfies the rho-condition. */
	ibk = 0;

	/*  ITER is number of iterations of the method. */
	iter = 0;

	forceb = FALSE_;
	L100: ++iter;

	/*  If AL is too small, terminate the line search, since convergence */
	/*  would be detected anyway by the calling routine. (X-convergence) */
	if (*al < alamin) {
		*al = allow;
		*fal = flow;
		*info = 3;
		return 0;
	}

	/*  Is AL getting too close on the A-bracket ? */
	/*  This is probably an indication of severe round off error near */
	/*  the solution. */
	if ((d__1 = a - *al, fabs(d__1)) / max(a, one) < alamin) {
		*al = allow;
		*fal = flow;
		*info = 3;
		return 0;
	}

	/*  Are we getting out of the right margin ? */
	if (*lim != 0 && *al > *xlim) {
		if (forceb) {
			*al = allow;
			*fal = flow;
			*info = 7;
			return 0;
		} else {
			*al = *xlim;
			forceb = TRUE_;
		}
	} else {
		forceb = FALSE_;
	}

	/*  Evaluate the objective function. When margins are present, small */
	/*  round off errors may force the point out of the margins. Take care. */
	meval_(&point[1], &dir[1], al, nod, &ixat[1], &marg[1], &xll[1], &xrl[1], &w[1], &c__1, fal);
	++(*nfc);
	if (*fal < flow) {
		flow = *fal;
		allow = *al;
	}
	if (*fal <= *ftarg) {
		/*  Function value less than target. */
		*info = 5;
		return 0;
	}

	if (*fal > *f0 + *al * *rho * g0 || *fal >= fa) {
		/*  Above the rho-line, or greater than the A-bracket. */
		++ibk;
		if (ibk == 1) {
			if (nob) {
				iside = 0;
			} else {
				iside = 0;
				if (b - a < 0.f) {
					iside = 1;
				}
			}

			/*  This is the first backtrack. Use a parabola. */
			qmin_(&a, &fa, &ga, al, fal, &nr, &xmin);
			if (nr == 0) {
				/*  Interpolation has failed. */
				*al = allow;
				*fal = flow;
				*info = 4;
				return 0;
			}
			aal = (d__1 = a - *al, fabs(d__1));
			if (iside == 0) {
				if (xmin < a + t1 * aal) {
					xmin = a + t1 * aal;
				}
			} else {
				if (xmin > a - t1 * aal) {
					xmin = a - t1 * aal;
				}
			}
		} else {

			/*  On the second and subsequent backtracks, use a cubic polynomial. */
			cmin2_(&a, &fa, &ga, al, fal, &b, &fb, &nr, &xmin);
			if (nr == 0) {
				qmin_(&a, &fa, &ga, al, fal, &nr, &xmin);
				if (nr == 0) {
					/*  Interpolation has failed. */
					*al = allow;
					*fal = flow;
					*info = 4;
					return 0;
				}
			}
			keept_(&a, al, &t1, &t2, &xmin);
		}
		b = *al;
		fb = *fal;
		nob = FALSE_;
	} else {
		/*  Below rho-line, less than A-bracket. */
		ibk = 0;
		/*  Evaluate the directional derivative. */
		i__1 = *nod;
		for (i__ = 1; i__ <= i__1; ++i__) {
			/* L30: */
			w[i__] = point[i__] + *al * dir[i__];
		}

		if (analytic == 1) {
			Objective_G(&w[1], *nod, &xtg[1]);
			++(*ngc);
		} else {
			IPRINT = 0;
			ftol = 1.0e-10;
			int jj;
			for (jj = 1; jj <= *nod; jj++) {
				if (w[jj] <= xll[jj])
					w[jj] = xll[jj] + 2.0 * (ftol);
				if (w[jj] >= xrl[jj])
					w[jj] = xrl[jj] - 2.0 * (ftol);
			}

			ondlga_(&w[1], nod, &xll[1], &xrl[1], &ftol, &iord, &IPRINT, &xtg[1], &NOC, &IERR);
			if (IERR != 0) {
				printf("Abort on numeric derivatives (3) %i \n", IERR);
				exit(0);
			}

			*nfc = *nfc + NOC;
		}

		vvmult_(&dir[1], &xtg[1], &gal, nod);
		/*         NFC = NFC+NOCD */
		/*         NGC = NGC+NGR */
		if (*method == 1) {
			/*  Algorithm S1. */
			if (gal >= *sigma * g0) {
				/*  Both line search conditions are satisfied. */
				*info = 0;
				return 0;
			}
		} else {
			/*  Algorithm S2. */
			if (fabs(gal) <= -(*sigma) * g0) {
				/*  Both line search conditions are satisfied. */
				*info = 0;
				return 0;
			}
		}

		/*  The gradient-criterion is not satisfied. */
		if (nob) {
			iside = 0;
			if (gal > 0.) {
				iside = 1;
			}
		} else {
			iside = 1;
			if ((b - a) * gal < 0.f) {
				iside = 0;
			}
		}

		/*  ISIDE should always be 0 when METHOD = 1. */
		if (*method == 1 && iside == 1) {
			printf("ABORT \n");
			exit(0);
		}
		/*     &          CALL MABORT('METHOD = 1, ISIDE = 1','LNS') */
		if (iside == 0) {
			if (gal > ga) {
				/*  The function is convex. */
				cmin_(&a, &fa, &ga, al, fal, &gal, &nr, &xmin);
				if (nr == 0) {
					/*  Quadratic extrapolation. Use the slopes at A, AL. */
					aa = (ga - gal) / ((a - *al) * 2.f);
					bb = ga - aa * 2 * a;
					xmin = -bb / (aa * 2);
				}
				keepe_(&a, al, &xmin, &b, &nob, &t3, &t4, &t5, &t6);
			} else {
				/*  The function is concave. */
				/*  Interpolating schemes are no good if the slopes indicate that this */
				/*  is a concave part of the function. (A maximum would actualy be */
				/*  found.) */
				if (nob) {
					/*  A right bracket has not been found. Try to establish a right */
					/*  bracket; use the right side of the allowed interval. */
					xmin = t4 * *al;
				} else {
					/*  A right bracket is available. */
					cmin2_(al, fal, &gal, &a, &fa, &b, &fb, &nr, &xmin);
					if (nr == 0) {
						qmin_(al, fal, &gal, &b, &fb, &nr, &xmin);
						if (nr == 0) {
							/*  Interpolation has failed. */
							*al = allow;
							*fal = flow;
							*info = 4;
							return 0;
						}
					}
					keepe_(&a, al, &xmin, &b, &nob, &t3, &t4, &t5, &t6);
				}
			}
		} else {
			/*  Other side. */
			cmin_(&a, &fa, &ga, al, fal, &gal, &nr, &xmin);
			if (nr == 0) {
				qmin_(al, fal, &gal, &a, &fa, &nr, &xmin);
				if (nr == 0) {
					/*  Interpolation has failed. */
					*al = allow;
					*fal = flow;
					*info = 4;
					return 0;
				}
			}
			keept_(&a, al, &t1, &t2, &xmin);
			b = a;
			fb = fa;
			nob = FALSE_;
		}
		a = *al;
		fa = *fal;
		ga = gal;
	}
	*al = xmin;

	/*  Perform another iteration. */
	if (iter < *maxit) {
		goto L100;
	}

	/*  The allowed number of line search iterations has been exhausted. */
	*al = allow;
	*fal = flow;
	*info = 1;

	return 0;
} /* lns_ */

/* Subroutine */int meval_(double *point, double *dir, double * al, int *nod, int *ixat, int *marg,
		double *xll, double *xrl, double *work, int *iopt, double *fal) {
	/* System generated locals */
	int i__1;
	double d__1, d__2;

	/* Local variables */
	int i__, ma;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--work;
	--xrl;
	--xll;
	--marg;
	--ixat;
	--dir;
	--point;

	/* Function Body */
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		if (ixat[i__] != 0) {
			work[i__] = point[i__] + *al * dir[i__];
			ma = marg[i__];
			if (ma == -1 || ma == 2) {
				/* Computing MAX */
				d__1 = work[i__], d__2 = xll[i__];
				work[i__] = max(d__1, d__2);
			}
			if (ma == 1 || ma == 2) {
				/* Computing MIN */
				d__1 = work[i__], d__2 = xrl[i__];
				work[i__] = min(d__1, d__2);
			}
		} else {
			work[i__] = point[i__];
		}
		/* L10: */
	}
	if (*iopt != 0) {
		Objective_F(&work[1], *nod, fal);
	}

	return 0;
} /* meval_ */

/*  --------------------------------------------------------------------- */
int ifconv_(double *val, double *oldval, double *eps) {
	/* System generated locals */
	int ret_val;
	double d__1, d__2;

	/* Local variables */
	double t1, t2, one;

	one = 1.f;
	t1 = (d__1 = *val - *oldval, fabs(d__1));
	/* Computing MAX */
	d__1 = one, d__2 = fabs(*val), d__1 = max(d__1, d__2), d__2 = fabs(*oldval);
	t2 = *eps * max(d__1, d__2);
	ret_val = 0;
	if (t1 <= t2) {
		ret_val = 1;
	}

	return ret_val;
} /* ifconv_ */

/*  --------------------------------------------------------------------- */
int igconv_(double *grad, double *point, int *nod, double *val, double *gtol) {
	/* System generated locals */
	int ret_val, i__1;
	double d__1, d__2, d__3, d__4;

	/* Local variables */
	int i__;
	double gc, one, gmax;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--point;
	--grad;

	/* Function Body */
	one = 1.f;
	gmax = 0.f;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* Computing MAX */
		d__3 = (d__1 = point[i__], fabs(d__1));
		/* Computing MAX */
		d__4 = fabs(*val);
		gc = (d__2 = max(d__3, one) * grad[i__] / max(d__4, one), fabs(d__2));
		gmax = max(gmax, gc);
		/* L88: */
	}
	ret_val = 0;
	if (gmax <= *gtol) {
		ret_val = 1;
	}

	return ret_val;
} /* igconv_ */

/*  --------------------------------------------------------------------- */
int ixconv_(double *point, double *oldpoi, int *nod, double *xtol) {
	/* System generated locals */
	int ret_val, i__1;
	double d__1, d__2, d__3;

	/* Local variables */
	int i__;
	double xc, one, xmax;

	/*  --------------------------------------------------------------------- */

	/* Parameter adjustments */
	--oldpoi;
	--point;

	/* Function Body */
	one = 1.f;
	xmax = 0.f;
	i__1 = *nod;
	for (i__ = 1; i__ <= i__1; ++i__) {
		/* Computing MAX */
		d__3 = (d__1 = oldpoi[i__], fabs(d__1));
		xc = (d__2 = (point[i__] - oldpoi[i__]) / max(d__3, one), fabs(d__2));
		xmax = max(xmax, xc);
		/* L98: */
	}
	ret_val = 0;
	if (xmax <= *xtol) {
		ret_val = 1;
	}

	return ret_val;
} /* ixconv_ */

/*  --------------------------------------------------------------------- */
int itconv_(double *val) {
	/* System generated locals */
	int ret_val;

	ret_val = 0;
	/*      IF (AIMON .AND. VAL.LE.TARGET) ITCONV = 1 */

	return ret_val;
} /* itconv_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int jrot_(double *a, int *i__, double *aa, double *bb, int *n, int *ld) {
	/* System generated locals */
	int a_dim1, a_offset, i__1;
	double d__1, d__2;

	/* Builtin functions */
	double d_sign(double *, double *), sqrt(double);

	/* Local variables */
	double c__;
	int j;
	double r__, s, w, y;
	int i1;
	double one;

	/* Parameter adjustments */
	a_dim1 = *ld;
	a_offset = 1 + a_dim1;
	a -= a_offset;

	/* Function Body */
	one = 1.f;
	if (*aa == 0.f) {
		c__ = 0.f;
		s = d_sign(&one, bb);
	} else {
		/* Computing 2nd power */
		d__1 = *aa;
		/* Computing 2nd power */
		d__2 = *bb;
		r__ = sqrt(d__1 * d__1 + d__2 * d__2);
		c__ = *aa / r__;
		s = *bb / r__;
	}

	i1 = *i__ + 1;
	i__1 = *n;
	for (j = *i__; j <= i__1; ++j) {
		y = a[*i__ + j * a_dim1];
		w = a[i1 + j * a_dim1];
		a[*i__ + j * a_dim1] = c__ * y - s * w;
		a[i1 + j * a_dim1] = s * y + c__ * w;
		/* L10: */
	}

	return 0;
} /* jrot_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int qmin_(double *x1, double *f1, double *g1, double *x2, double *f2, int *nr,
		double *xmin) {
	double b, df, dx;

	dx = *x2 - *x1;
	df = *f2 - *f1;
	b = (df / dx - *g1) / dx;

	if (b > 0.) {
		*xmin = *x1 - *g1 / (b * 2);
		*nr = 1;
	} else {
		*nr = 0;
	}

	return 0;
} /* qmin_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int cmin_(double *x1, double *f1, double *g1, double *x2, double *f2, double *g2,
		int *nr, double *xmin) {
	/* System generated locals */
	double d__1, d__2;

	/* Builtin functions */
	double sqrt(double);

	/* Local variables */
	double a, b, d__, h__;

	h__ = *x2 - *x1;
	/* Computing 2nd power */
	d__1 = h__;
	/* Computing 3rd power */
	d__2 = h__;
	a = (*g2 + *g1) / (d__1 * d__1) - (*f2 - *f1) * 2 / (d__2 * (d__2 * d__2));
	/* Computing 2nd power */
	d__1 = h__;
	b = ((*f2 - *f1) * 3 - h__ * (*g2 + *g1 * 2)) / (d__1 * d__1);
	/* Computing 2nd power */
	d__1 = b;
	d__ = d__1 * d__1 - a * 3 * *g1;
	if (d__ >= 0.) {
		if (a != 0.) {
			*xmin = *x1 + (-b + sqrt(d__)) / (a * 3);
			*nr = 1;
		} else {
			if (b > 0.) {
				*xmin = *x1 - *g1 / (b * 2);
				*nr = 1;
			} else {
				*nr = 0;
			}
		}
	} else {
		*nr = 0;
	}

	return 0;
} /* cmin_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int qmin2_(double *x1, double *f1, double *x2, double *f2, double *x3, double *f3,
		int *nr, double *xmin) {
	/* System generated locals */
	double d__1, d__2;

	/* Local variables */
	double a, d1, d2, d3, df1, df2, den, anu;

	d1 = *x2 - *x1;
	d2 = *x1 - *x3;
	d3 = *x3 - *x2;

	df1 = *f2 - *f1;
	df2 = *f3 - *f2;

	a = (df1 / d1 - df2 / d3) / d2;
	if (a > 0.f) {
		/* Computing 2nd power */
		d__1 = d3;
		/* Computing 2nd power */
		d__2 = d1;
		anu = df1 * (d__1 * d__1) + df2 * (d__2 * d__2);
		den = df1 * d3 - df2 * d1;
		*xmin = *x2 + anu / (den * 2.f);
		*nr = 1;
	} else {
		*nr = 0;
	}

	return 0;
} /* qmin2_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int cmin2_(double *x1, double *f1, double *g1, double *x2, double *f2, double *x3,
		double *f3, int *nr, double *xmin) {
	/* System generated locals */
	double d__1;

	/* Builtin functions */
	double sqrt(double);

	/* Local variables */
	double a, b, d__, w1, w2;

	/* Computing 2nd power */
	d__1 = *x2 - *x1;
	w1 = (*f2 - *f1 - (*x2 - *x1) * *g1) / (d__1 * d__1);
	/* Computing 2nd power */
	d__1 = *x3 - *x1;
	w2 = (*f3 - *f1 - (*x3 - *x1) * *g1) / (d__1 * d__1);
	a = (w2 - w1) / (*x3 - *x2);
	b = (*x2 - *x1) * w2 / (*x2 - *x3) - (*x3 - *x1) * w1 / (*x2 - *x3);
	/* Computing 2nd power */
	d__1 = b;
	d__ = d__1 * d__1 - a * 3 * *g1;
	if (d__ >= 0.) {
		if (a != 0.) {
			*xmin = *x1 + (-b + sqrt(d__)) / (a * 3);
			*nr = 1;
		} else {
			if (b > 0.) {
				*xmin = *x1 - *g1 / (b * 2);
				*nr = 1;
			} else {
				*nr = 0;
			}
		}
	} else {
		*nr = 0;
	}

	return 0;
} /* cmin2_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int keept_(double *a, double *al, double *t1, double *t2, double *xmin) {
	/* System generated locals */
	double d__1;

	/* Local variables */
	double aalen, xleft, xright;

	aalen = (d__1 = *a - *al, fabs(d__1));

	if (*a < *al) {
		xleft = *a + *t1 * aalen;
		xright = *al - *t2 * aalen;
	} else {
		xleft = *al + *t2 * aalen;
		xright = *a - *t1 * aalen;
	}

	if (*xmin < xleft) {
		*xmin = xleft;
	} else if (*xmin > xright) {
		*xmin = xright;
	}

	return 0;
} /* keept_ */

/*  --------------------------------------------------------------------- */
/* Subroutine */int keepe_(double *a, double *al, double *xmin, double *b, int *nob, double *t3,
		double *t4, double *t5, double *t6) {
	/* System generated locals */
	double d__1;

	/* Local variables */
	double xleft, alblen, xright;

	alblen = (d__1 = *al - *b, fabs(d__1));
	if (*a < *al) {
		if (*nob) {
			xleft = *t3 * *al;
			xright = *t4 * *al;
		} else {
			xleft = *al + *t5 * alblen;
			xright = *b - *t6 * alblen;
		}
	} else {
		xleft = *b + *t6 * alblen;
		xright = *al - *t5 * alblen;
	}

	if (*xmin < xleft) {
		*xmin = xleft;
	} else if (*xmin > xright) {
		*xmin = xright;
	}

	return 0;
} /* keepe_ */

