#include <math.h>
#include <omp.h>
#ifndef max
#define max(a,b)	((a)>(b)?(a):(b))
#endif

extern void Objective_F(double X[], int N, double *F);

static double objective(double *x, int n)
{
	double tmp;
	Objective_F(x, n, &tmp);
	return tmp;
}


static void slave(double *xx, double *pfac, double *pstep, double *ph, int *pn, int *pipos, double *res)
{
	double fac = *pfac;	
	double step = *pstep;
	double h = *ph;
	int n = *pn;
	int ipos = *pipos;

	int i;
	double temp[1000];
	double val;

	for (i=0; i<n; i++) temp[i] = xx[i];
	temp[ipos] = temp[ipos] + step*h;
	val = fac*objective(temp, n);

	*res = val;
}

void ondl1f(double *x, int n, double *xl, double *xu, double feps, int iprint, double *g, int *noc)
{
	double xx[1000], hh[1000];
	int ifb[1000];
	const double one = 1.0;
	int i;

	double con;
	double fac, f0;
	double hi;
	double step;
	int ipos;
	double xi, xp, xm;

	con = sqrt(feps);

	for (i=0; i<n; i++) {
		g[i] = 0.0;
		xx[i] = x[i];
	}

	step = 0.0;
	fac = one;
	ipos = 0;
	f0 = 0.0;
	hi = 0.0;

	slave(xx, &fac, &step, &hi, &n, &ipos, &f0);

	*noc = 1;

	for (i=0; i<n; i++) {
		xi = x[i];
		hi = con*max(fabs(xi), one);
		hh[i] = hi;
		xp = xi+hi;
		xm = xi-hi;
		if (xp<xu[i]) {
			x[i]=xp;
			step=one;
			fac=one;
			#pragma omp task shared(xx,g) firstprivate(fac,step,hi,n,i) 
			{
				slave(xx,&fac,&step,&hi,&n,&i,&g[i]);
			}
			(*noc)++;
			ifb[i]=1;
		} else if(xm>xl[i]) {
			x[i] = xm;
			step = -one;
			fac = one;
			#pragma omp task shared(xx,g) firstprivate(fac,step,hi,n,i) 
			{
				slave(xx,&fac,&step,&hi,&n,&i,&g[i]);
			}
			(*noc)++;
			ifb[i]=-1;
		} else {
			g[i] = 0.0;
		}
		x[i] = xi;
	}
	#pragma omp taskwait

	for (i=0;i<n;i++) {
		if (ifb[i]==1) {
			g[i] = (g[i]-f0) / hh[i];
		} else if (ifb[i]==-1) {
			g[i] = (f0-g[i]) / hh[i];
		}
	}
}
