#include <math.h>
#include <omp.h>
#include <stdio.h>
#ifndef max
#define max(a,b)	((a)>(b)?(a):(b))
#endif
extern void Objective_F(double X[], int N, double *F);

static double objective(double *x, int n)
{
	double tmp;
	Objective_F(x, n, &tmp);
	return tmp;
}


static void slave(double *xx, double *pfac, double *pstep, double *ph, int *pn, int *pipos, double *res)
{
	double fac = *pfac;
	double step = *pstep;
	double h = *ph;
	int n = *pn;
	int ipos = *pipos;

	int i;
	double temp[1000];	/* max problem dimension */
	double val;

	for (i=0; i<n; i++) temp[i] = xx[i];
	temp[ipos] = temp[ipos] + step*h;
	val = fac*objective(temp, n);

	*res = val;
}


void ondl1q(double *x, int n, double *xl, double *xu, double feps, int iprint, double *g, int *noc)
{
	double fh[1000], fhh[1000];
	double xx[1000], hh[1000];
	int ifb[1000];
	const double one = 1.0;
	int i;
	int nof0;

	double con;
	double f0;

	con = pow(feps, 1.0/3.0);
	*noc = 0;
	nof0 = 1;
	f0 = 0.0;

	double fac;
	double hi;
	double step;
	double xi, xp, xm, xpp, xmm;

	for (i=0; i<n; i++) {
		g[i] = 0.0;
		xx[i] = x[i];
		ifb[i] = 0;
	}

	for (i=0; i<n; i++) {
		xi = x[i];
		hi = con*max(fabs(xi), one);
		hh[i] = hi;
		xp = xi+hi;
		xm = xi-hi;
		if ((xm>xl[i])&&(xp<xu[i])) { 
			x[i] = xp;
//C			FP = F(X,N) -> FH
			step = 1.0;
			fac = 1.0;
			#pragma omp task shared(xx,fh) firstprivate(fac,step,hi,n,i)
			{
				slave(xx,&fac,&step,&hi,&n,&i,&fh[i]);
			}
			x[i] = xm;
//C			FM = F(X,N) -> FHH
			step = -1.0;
			fac = 1.0;
			#pragma omp task shared(xx,fhh) firstprivate(fac,step,hi,n,i)
			{
				slave(xx,&fac,&step,&hi,&n,&i,&fhh[i]);
			}
//c			G(I) = (FP-FM)/(2.0D0*HI)
			*noc = *noc+2;
		} else if (xm<xl[i]) {
			xpp = xi+2.0*hi;
			if (xpp < xu[i]) {
				if (nof0) {
//C					F0 = F(X,N)
					step = 0.0;
					fac = 1.0;
					#pragma omp task shared(xx,f0) firstprivate(fac,step,hi,n,i)
					{
						slave(xx,&fac,&step,&hi,&n,&i,&f0);
					}

					nof0 = 0;
				}
				x[i] = xp;
//C				FP = F(X,N) -> FH
				step = 1.0;
				fac = 1.0;
				#pragma omp task shared(xx,fh) firstprivate(fac,step,hi,n,i)
				{
					slave(xx,&fac,&step,&hi,&n,&i,&fh[i]);
				}

				x[i] = xpp;
//c				FPP = F(X,N) -> FHH
				step = 2.0;
				fac = 1.0;
				#pragma omp task shared(xx,fhh) firstprivate(fac,step,hi,n,i)
				{
					slave(xx,&fac,&step,&hi,&n,&i,&fhh[i]);
				}
//c				G(I) = (4.0D0*FP-3.0D0*F0-FPP)/(2.0D0*HI)
				*noc = *noc+3;
				ifb[i] = 1;
			} else {
				g[i] = 0.0;
			}
		} else if (xp>xu[i]) {
			xmm = xi - 2.0*hi;
			if (xmm>xl[i]) {
				if (nof0) {
//c					F0 = F(X,N)
					step = 0.0;
					fac = 1.0;
					#pragma omp task shared(xx,f0) firstprivate(fac,step,hi,n,i)
					{
						slave(xx,&fac,&step,&hi,&n,&i,&f0);
					}
					nof0 = 0;
					*noc = *noc+1;
				}
				x[i] = xm;
//c				FM = F(X,N) -> FH
				step = -1.0;
				fac = 1.0;
				#pragma omp task shared(xx,fh) firstprivate(fac,step,hi,n,i)
				{
					slave(xx,&fac,&step,&hi,&n,&i,&fh[i]);
				}
				x[i] = xmm;
//C				FMM = F(X,N) -> FHH
				step = -2.0;
				fac = 1.0;
				#pragma omp task shared(xx,fhh) firstprivate(fac,step,hi,n,i)
				{
					slave(xx,&fac,&step,&hi,&n,&i,&fhh[i]);
				}
//C				G(I) = -(4.0D0*FM-3.0D0*F0-FMM)/(2.0D0*HI)
				*noc = *noc+2;
				ifb[i] = -1;
			} else {
				g[i] = 0.0;
			}
		} else {
			g[i] = 0.0;
		}
		x[i] = xi;
	}
	#pragma omp taskwait

	for (i=0;i<n;i++) {	
		if (ifb[i]==1) {
			g[i] =  (4.0*fh[i]-3.0*f0-fhh[i])/(2.0*hh[i]);
		} else if (ifb[i]==-1) {
			g[i] = -(4.0*fh[i]-3.0*f0-fhh[i])/(2.0*hh[i]);
		} else {
			g[i] = (fh[i]-fhh[i])/(2.0*hh[i]);
		}
	}
}

