#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netdb.h>
#include "objective.h"

void Objective_F (double X[], int N, double *F);
void Objective_G (double X[], int N, double G[]);

int send_all(int fd, void *buffer, unsigned int size);
int recv_all(int fd, void *buffer, unsigned int size);

extern  int sock_array[MAXOMP];


void Objective_F (double X[], int N, double *F)
{
	int j, me;
	double *datain, dataout[1];
	int sizein = (N+1)*sizeof(double);
	int sizeout = 1*sizeof(double);
	datain = (double *) malloc(sizein);


	for (j=0; j<N; j++)
		datain[j] = X[j];
	datain[N] = 0.0;

	me = omp_get_thread_num();	
	/* calling tinker */
	send_all(sock_array[me], datain, sizein);
	recv_all(sock_array[me], dataout, sizeout);

	*F = dataout[0];

	free(datain);
	FEVALCOUNT;
}

void Bounds_F(double *xl, double *xr, int N)
{
	int i;
	for (i=0; i<N; i++)
	{
		xl[i] = -180.0;
		xr[i] =  180.0;
	}
}



void Objective_G(double X[], int N, double G[])
{
	int j, me;
	double *datain, *dataout;
	int sizein = (N+1)*sizeof(double);
	int sizeout = N*sizeof(double);
	datain = (double *) malloc(sizein);
	dataout= (double *) malloc(sizeout);

	for (j=0; j<N; j++)
		datain[j] = X[j];
	datain[N] = 1.0;

	me = omp_get_thread_num();
	/* calling tinker */
	send_all(sock_array[me], datain, sizein);
	recv_all(sock_array[me], dataout, sizeout);

	for (j=0; j<N; j++)
		G[j] = dataout[j];

	free(datain);
	free(dataout);
}


/*************************************************************************/
/**************************   HELPER ROUTINES   **************************/
/*************************************************************************/

int recv_all(int fd, void *buffer, unsigned int size)
{
	int result;
	unsigned int s=size;
	char *pos = (char*)buffer;


	do {
		result=recv(fd,pos,s,0);
		if((result!=-1)&&(result>0)) {
			s -= result;
			pos += result;
		}
		else 
			return result; //-1;
	} while (s>0);
	return size;
}


int send_all(int fd, void *buffer, unsigned int size)
{
	int result;
	unsigned int s=size;
	char *pos = (char*)buffer;

	do {
		result=send(fd,pos,s,0);
		if((result!=-1)&&(result>0)) {
			s -= result;
			pos += result;
		}
		else return result; //-1;
	} while (s>0);
	return size;
}
