#include "util.h" 
#include <stdio.h>
#include <unistd.h>

char SOCK_PATH[256];

static void client()
{
	int sock;
	struct sockaddr_un server_addr;
	int servlen;
	int i, j;
	double t1, t2;

	double datain[DIM+1], dataout[1];
	int sizein = (DIM+1)*sizeof(double);
	int sizeout = 1*sizeof(double);


	/* Create a socket */
	sock = socket(AF_UNIX, SOCK_STREAM, 0);

	/* Specify the server */
	bzero((char *)&server_addr, sizeof(server_addr));
	server_addr.sun_family = AF_UNIX;
	strcpy(server_addr.sun_path, SOCK_PATH);
	servlen = sizeof(server_addr.sun_family) + strlen(server_addr.sun_path);

	/* Connect to the server */
	if (connect(sock, (struct sockaddr *)&server_addr, servlen) < 0) {
		perror("connect");
		exit(1);
	}

        t1 = my_gettime();
        for (i = 0; i < ITERS; i++) {
		/* Send a descriptor */
		for (j=0; j<DIM; j++)
			datain[j] = 360.0*((float)rand()/(float)RAND_MAX)-180.0;
		datain[DIM] = 0.0;
	
		/* Send the data */
		send_all(sock, datain, sizein);

		recv_all(sock, dataout, sizeout);
#if DBG
                printf("response = %lf\n", dataout[0]);
#endif
        }
        t2 = my_gettime();
        printf("CLIENT: Elapsed time for %d iters = %.3lf us (%.3lf us/iter)\n", ITERS, 1e6*(t2-t1), 1e6*(t2-t1)/ITERS);

	/* Close down and exit */
	close(sock);
}

static void client_grad()
{
        int sock;
        struct sockaddr_un server_addr;
        int servlen;
        int i, j;
        double t1, t2;

        double datain[DIM+1], dataout[DIM];
        int sizein = (DIM+1)*sizeof(double);
        int sizeout = DIM*sizeof(double);


        /* Create a socket */
        sock = socket(AF_UNIX, SOCK_STREAM, 0);

        /* Specify the server */
        bzero((char *)&server_addr, sizeof(server_addr));
        server_addr.sun_family = AF_UNIX;
        strcpy(server_addr.sun_path, SOCK_PATH);
        servlen = sizeof(server_addr.sun_family) + strlen(server_addr.sun_path);

        /* Connect to the server */
        if (connect(sock, (struct sockaddr *)&server_addr, servlen) < 0) {
                perror("connect");
                exit(1);
        }

        t1 = my_gettime();
        for (i = 0; i < ITERS; i++) {
                /* Send a descriptor */
                for (j=0; j<DIM; j++)
                        datain[j] = 360.0*((float)rand()/(float)RAND_MAX)-180.0;
		datain[DIM] = 1.0;

                /* Send the data */
                send_all(sock, datain, sizein);

                recv_all(sock, dataout, sizeout);
#if DBG
                printf("response = %lf %lf %lf\n", dataout[0], dataout[1], dataout[2]);
#endif
        }
        t2 = my_gettime();
        printf("CLIENT: Elapsed time for %d iters = %.3lf us (%.3lf us/iter)\n", ITERS, 1e6*(t2-t1), 1e6*(t2-t1)/ITERS);

        /* Close down and exit */
        close(sock);
}

int main(int argc, char *argv[])
{
	int fg;
	if (argc != 4) {
		printf("Usage: %s <sock_path> <id> <feval=0/geval=1>\n", argv[0]);
		exit(1);
	}

	fg = atoi(argv[3]);	
	sprintf(SOCK_PATH, "%s%s\n", argv[1], argv[2]);
	printf("SOCK_PATH=%s\n", SOCK_PATH);

	if (fg ==0)
		client();
	else
		client_grad();

	return 0;
}


